/***************************************************************************
                          audio.cpp  -  description
                             -------------------
    begin                : Mon Oct 13 2003
    copyright            : (C) 2003 by Guilherme Fortunato
    email                : guilherme@linksat.com.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "audio.h"

extern char disk_out_file[1024];
char devices[256][3];
short *audio_bufs[2] = { NULL, NULL };
uint n_bytes = 0;
int cur_buf = 0;

static int var_int;
static int dev;

pthread_t snd_thread;
void* ( *snd_thread_func )( void * );

static uint buffer_time;
static snd_pcm_t *alsa_device;
static int oss_device;
static std::fstream disk_device;

static const char *devices_names[3] = { "default", "surround40", "surround51" };

Plugin_Audio_Driver::Plugin_Audio_Driver()
{
	drv_init[0] = drv_init[1] = drv_init[2] = drv_init[3] = false;
}

Plugin_Audio_Driver::~Plugin_Audio_Driver()
{
	shutdownDriver();
	deallocBuffers();
}

void Plugin_Audio_Driver::allocBuffers()
{
	audio_bufs[0] = new( short[buffer_size] );
	audio_bufs[1] = new( short[buffer_size] );
}

void Plugin_Audio_Driver::deallocBuffers()
{
	delete[] audio_bufs[0];
	audio_bufs[0] = NULL;
	delete[] audio_bufs[1];
	audio_bufs[1] = NULL;
}

void Plugin_Audio_Driver::setDriver( SOUND_DRIVER drv )
{
	snd_drv = drv;
}

void Plugin_Audio_Driver::setSync( bool sync )
{
	sync_fix = sync;
}

void Plugin_Audio_Driver::setQuery( bool query )
{
	query_hardware = query;
}

void Plugin_Audio_Driver::setRate( int new_rate )
{
	rate = static_cast<uint>( new_rate );
	configureDriver( true );
}

void Plugin_Audio_Driver::setVolumes( int left, int right )
{
	vol_left = static_cast<uint>( left );
	vol_right = static_cast<uint>( right );
}

void Plugin_Audio_Driver::setChannels( int channels )
{
	n_channels = static_cast<uint>( channels );
}

void Plugin_Audio_Driver::setBufferSize( int size )
{
	buffer_size = static_cast<snd_pcm_uframes_t>( size );
}

void Plugin_Audio_Driver::setPeriodSize( int size )
{
	period_size = static_cast<snd_pcm_uframes_t>( size );
}

const bool Plugin_Audio_Driver::getDriver( SOUND_DRIVER drv ) const
{
	return static_cast<bool>( ( drv == snd_drv ) );
}

const SOUND_DRIVER Plugin_Audio_Driver::getDriver() const
{
	return snd_drv;
}

const bool Plugin_Audio_Driver::getSync() const
{
	return sync_fix;
}

const bool Plugin_Audio_Driver::getQuery() const
{
	return query_hardware;
}

const uint Plugin_Audio_Driver::getVolumeLeft() const
{
	return vol_left;
}

const uint Plugin_Audio_Driver::getVolumeRight() const
{
	return vol_right;
}

const uint Plugin_Audio_Driver::getVolumeMax() const
{
	return vol_max;
}

const uint Plugin_Audio_Driver::getChannels() const
{
	return n_channels;
}

const uint Plugin_Audio_Driver::getBufferSize() const
{
	return buffer_size;
}

const uint Plugin_Audio_Driver::getPeriodSize() const
{
	return period_size;
}

void Plugin_Audio_Driver::setBufferTime( int time )
{
	buffer_time = static_cast<uint>( time );
}

const uint Plugin_Audio_Driver::getBufferTime() const
{
	return buffer_time;
}

bool Plugin_Audio_Driver::initAlsa()
{
	switch ( n_channels ) {
		case 4:
			dev = 1; break;
		case 5: case 6:
			dev = 2; break;
		default:
			dev = 0;
	}

	if ( snd_pcm_open( &alsa_device, devices_names[dev], SND_PCM_STREAM_PLAYBACK, sync_fix == true ? SND_PCM_NONBLOCK : 0 ) < 0 ) return false;
	snd_pcm_nonblock( alsa_device, sync_fix );
	snd_pcm_hw_params_malloc( &hw_params );
	snd_pcm_hw_params_any( alsa_device, hw_params );
	snd_pcm_access_mask_t *mask = static_cast<snd_pcm_access_mask_t *>( malloc( snd_pcm_access_mask_sizeof() ) );
	snd_pcm_access_mask_none( mask );
	snd_pcm_access_mask_set( mask, SND_PCM_ACCESS_MMAP_INTERLEAVED );
	snd_pcm_access_mask_set( mask, SND_PCM_ACCESS_MMAP_COMPLEX );
	snd_pcm_access_mask_set( mask, SND_PCM_ACCESS_MMAP_NONINTERLEAVED );
	snd_pcm_hw_params_set_access_mask( alsa_device, hw_params, mask );
	free( ( void * )mask );
	snd_pcm_hw_params_set_access( alsa_device, hw_params, SND_PCM_ACCESS_MMAP_INTERLEAVED );
	snd_pcm_hw_params_set_format( alsa_device, hw_params, SND_PCM_FORMAT_S16_LE );
	snd_pcm_sw_params_malloc( &sw_params );

	snd_mixer_open( &mixer, 0 );
	snd_mixer_attach( mixer, devices_names[0] );

	//snd_pcm_hw_params_get_channels_max( hw_params, &( uint )var_int );
	var_int = 2;
	if ( ( var_int & dev ) == 2 ) return false;

	snd_mixer_selem_register( mixer, NULL, NULL );
	snd_mixer_load( mixer );
	mixer_elem = snd_mixer_first_elem( mixer );
	snd_mixer_selem_get_playback_volume_range( mixer_elem, ( long * )&var_int, ( long * )&vol_max );
	return true;
}

bool Plugin_Audio_Driver::initOSS()
{
	if ( ( oss_device = open( "/dev/sound/audio", O_WRONLY ) ) < 0 )
		if ( ( oss_device = open( "/dev/audio", O_WRONLY ) ) < 0 ) return false;
	snd_thread_func = oss_thread;
	return true;
}

bool Plugin_Audio_Driver::initDiskWriter()
{
	disk_device.open( disk_out_file, std::ios::binary | std::ios::trunc | std::ios::out );
	if ( disk_device.fail() ) return false;
	snd_thread_func = disk_thread;
	return true;
}

void Plugin_Audio_Driver::initDriver( bool force_shutdown )
{
	if ( drv_init[snd_drv] && !force_shutdown ) return;
	shutdownDriver();
	switch ( snd_drv ) {
		case ALSA_LIB:
			var_int = initAlsa(); break;
		case ALSA_OSS:
			var_int = initOSS(); break;
		case DISK_WRITER:
   			var_int = initDiskWriter();
		default: break;
	}
	drv_init[snd_drv] = static_cast<bool>( var_int );
}

void Plugin_Audio_Driver::configureDriver( bool leave_here )
{
	switch ( snd_drv ) {
		case ALSA_LIB:
			if ( !leave_here ) rate = 32000;
			snd_pcm_hw_params_set_rate_near( alsa_device, hw_params, &rate, NULL );
			if ( leave_here ) return;

			if ( sync_fix ) {
				snd_thread_func = alsa_thread_method1;
				xfer = 4;
			}
			else {
				snd_thread_func = alsa_thread_method2;
				xfer = 1;
			}

			snd_pcm_hw_params_set_channels( alsa_device, hw_params, n_channels );
			if ( query_hardware ) {
				snd_pcm_hw_params_get_buffer_time( hw_params, &buffer_time, 0 );
				snd_pcm_hw_params_get_buffer_size_max( hw_params, &buffer_size );
				period_size = buffer_size >> 4;
				//snd_pcm_hw_params_get_period_size( hw_params, &period_size, NULL );
			}
			var_int = snd_pcm_hw_params_set_buffer_time( alsa_device, hw_params, buffer_time, SND_PCM_STREAM_PLAYBACK );
			snd_pcm_hw_params_set_buffer_size( alsa_device, hw_params, buffer_size );
			snd_pcm_hw_params_set_period_size( alsa_device, hw_params, period_size, SND_PCM_STREAM_PLAYBACK );
			snd_pcm_hw_params_set_period_time( alsa_device, hw_params, buffer_time / 4, SND_PCM_STREAM_PLAYBACK );
			snd_pcm_hw_params( alsa_device, hw_params );
			if ( vol_left > vol_max ) vol_left = vol_max;
			if ( vol_right > vol_max ) vol_right = vol_max;
			if ( n_channels == 1 )
				snd_mixer_selem_set_playback_volume( mixer_elem, SND_MIXER_SCHN_MONO, vol_left );
			else {
				snd_mixer_selem_set_playback_volume( mixer_elem, SND_MIXER_SCHN_FRONT_LEFT, vol_left );
				snd_mixer_selem_set_playback_volume( mixer_elem, SND_MIXER_SCHN_FRONT_RIGHT, vol_right );
				if ( n_channels >= 4 ) {
					snd_mixer_selem_set_playback_volume( mixer_elem, SND_MIXER_SCHN_REAR_LEFT, vol_left );
					snd_mixer_selem_set_playback_volume( mixer_elem, SND_MIXER_SCHN_REAR_RIGHT, vol_right );
					if ( n_channels >= 5 )
						snd_mixer_selem_set_playback_volume( mixer_elem, SND_MIXER_SCHN_WOOFER, vol_left );
				}
			}
			snd_pcm_sw_params_current( alsa_device, sw_params );
			snd_pcm_sw_params_set_xfer_align( alsa_device, sw_params, xfer ); // 1 for block; 4 for non-block
			if ( sync_fix ) {
				snd_pcm_sw_params_set_avail_min( alsa_device, sw_params, 0 );
				snd_pcm_sw_params_set_start_threshold( alsa_device, sw_params, period_size );
				snd_pcm_sw_params_set_stop_threshold( alsa_device, sw_params, buffer_size / 4 );
			}
			snd_pcm_sw_params_set_sleep_min( alsa_device, sw_params, period_size );
			snd_pcm_sw_params( alsa_device, sw_params );
			break;
		case ALSA_OSS:
			var_int = 1;
			ioctl( oss_device, SNDCTL_DSP_STEREO, &var_int );
			var_int = vol_left * 2;
			ioctl( oss_device, SOUND_MIXER_WRITE_VOLUME, &var_int );
			ioctl( oss_device, SOUND_PCM_WRITE_CHANNELS, &n_channels );
			ioctl( oss_device, SOUND_PCM_WRITE_RATE, &rate );
			if ( query_hardware ) {
				ioctl( oss_device, SNDCTL_DSP_GETBLKSIZE, &buffer_size );
				period_size = buffer_size >> 4;
			}
			//var_int = 0x30018;
   			//ioctl( oss_device, SNDCTL_DSP_SETFRAGMENT, &var_int );
			ioctl( oss_device, SNDCTL_SEQ_THRESHOLD, &buffer_size );
			var_int = AFMT_S16_LE;
			ioctl( oss_device, SNDCTL_DSP_SETFMT, &var_int );
			break;
		default: break;
	}
	if ( audio_bufs[0] == NULL ) allocBuffers();
	#if DEBUG
	std::cerr << snd_drv << std::endl;
	std::cerr << n_bytes << std::endl;
	std::cerr << rate << std::endl;
	std::cerr << n_channels << std::endl;
	std::cerr << dev << std::endl;
	std::cerr << buffer_size << std::endl;
	std::cerr << buffer_time << std::endl;
	std::cerr << period_size << std::endl;
	std::cerr << query_hardware << std::endl;
	std::cerr << sync_fix << std::endl;
	std::cerr << vol_left << std::endl;
	std::cerr << vol_right << std::endl;
	#endif
}

void Plugin_Audio_Driver::shutdownDriver()
{
	if ( alsa_device != NULL ) {
		snd_mixer_close( mixer );
		snd_pcm_hw_params_free( hw_params );
		snd_pcm_sw_params_free( sw_params );
		snd_pcm_close( alsa_device );
		alsa_device = NULL;
		drv_init[ALSA_LIB] = false;
	}
	else if ( oss_device >= 0 ) {
		close( oss_device );
		oss_device = -1;
		drv_init[ALSA_OSS] = false;
	}
	else if ( disk_device.is_open() ) {
		disk_device.close();
		drv_init[DISK_WRITER] = false;
	}
}

void Plugin_Audio_Driver::prepare()
{
	switch ( snd_drv ) {
		case ALSA_LIB:
			snd_pcm_prepare( alsa_device ); break;
		case ALSA_OSS:
			ioctl( oss_device, SNDCTL_DSP_RESET ); break;
		default: break;
	}
}

void Plugin_Audio_Driver::pause()
{
	switch( snd_drv ) {
		case ALSA_LIB:
			snd_pcm_drop( alsa_device ); break;
		case ALSA_OSS:
			ioctl( oss_device, SNDCTL_DSP_SYNC ); break;
		case DISK_WRITER:
			shutdownDriver();
		default: break;
	}
}

void Plugin_Audio_Driver::getHardware( int &ret_val )
{
	snd_ctl_t *handle;
	snd_ctl_card_info_t *info;
	snd_pcm_info_t *pcminfo;
	snd_ctl_card_info_alloca( &info );
	snd_pcm_info_alloca( &pcminfo );
	char name[20];
	int n_cards = -1;
	ret_val = 0;

	if ( snd_card_next( &n_cards ) < 0 || n_cards < 0 )
		return;
	while ( n_cards >= 0 ) {
		sprintf( name, "hw:%d", n_cards );
		if ( snd_ctl_open( &handle, name, 0 ) < 0 ) {
			#ifdef DEBUG
			std::cerr << "control open (" << n_cards << ") failed." << std::endl;
			#endif
			continue;
		}
		if ( snd_ctl_card_info( handle, info ) < 0 ) {
			#ifdef DEBUG
			std::cerr << "control hardware info (" << n_cards << ") failed." << std::endl;
			#endif
			snd_ctl_close( handle );
			continue;
		}
		var_int = -1;
		while ( true ) {
			#ifdef DEBUG
			if (
			#endif
			snd_ctl_pcm_next_device( handle, &var_int )
			#ifndef DEBUG
			;
			#endif
			#ifdef DEBUG
			< 0 )
			std::cerr << "snd_ctl_pcm_next_device" << std::endl;
			#endif

			if ( var_int < 0 ) break;
			snd_pcm_info_set_device( pcminfo, var_int );
			snd_pcm_info_set_subdevice( pcminfo, 0 );
			snd_pcm_info_set_stream( pcminfo, SND_PCM_STREAM_PLAYBACK );
			if ( snd_ctl_pcm_info( handle, pcminfo ) < 0 ) continue;
			sprintf( devices[2 - ret_val], "card %i: %s [%s], device %i: %s [%s]",
				n_cards, snd_ctl_card_info_get_id( info ), snd_ctl_card_info_get_name( info ),
				var_int, snd_pcm_info_get_id( pcminfo ), snd_pcm_info_get_name( pcminfo ) );
			ret_val++;
		}
		snd_ctl_close( handle );
		if ( snd_card_next( &n_cards ) < 0 ) {
			#ifdef DEBUG
			std::cerr << "No more cards: snd_card_next" << std::endl;
			#endif
			break;
		}
	}
}

void Plugin_Audio_Driver::setDefaults()
{
	dev = 0;
	vol_left = vol_right = 50;
	vol_max = 100;
	n_channels = 2;
	buffer_time = 1000000;
	buffer_size = 16384;
	rate = 32000;
	period_size = 1024;
	query_hardware = true;
	sync_fix = false;
	snd_drv = ALSA_LIB;
}

void *Plugin_Audio_Driver::alsa_thread_method1( void *emp )
{
	if ( snd_pcm_state( alsa_device ) == SND_PCM_STATE_RUNNING )
		snd_pcm_wait( alsa_device, buffer_time );
	int write_res;
	write_res = snd_pcm_mmap_writei( alsa_device, audio_bufs[cur_buf], snd_pcm_bytes_to_frames( alsa_device, n_bytes ) );
	if ( write_res == -EPIPE ) {
		snd_pcm_prepare( alsa_device );
		return NULL;
	}
	else if ( write_res == -EAGAIN || write_res < int( n_bytes ) )
		snd_pcm_wait( alsa_device, buffer_time );
	return NULL;
}

void *Plugin_Audio_Driver::alsa_thread_method2( void *emp )
{
	if ( snd_pcm_mmap_writei( alsa_device, audio_bufs[cur_buf], snd_pcm_bytes_to_frames( alsa_device, n_bytes ) ) < 0 )
		snd_pcm_prepare( alsa_device );
	return NULL;
}

void *Plugin_Audio_Driver::oss_thread( void *emp )
{
	write( oss_device, audio_bufs[cur_buf], n_bytes );
	return NULL;
}

void *Plugin_Audio_Driver::disk_thread( void *emp )
{
	disk_device.write( ( char * )( audio_bufs[cur_buf] ), n_bytes );
	return NULL;
}
