/***************************************************************************
                          audio.h  -  description
                             -------------------
    begin                : Sat Oct 11 2003
    copyright            : (C) 2003 by Guilherme Fortunato
    email                : guilherme@linksat.com.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PLUGIN_AUDIO_DRIVER_H
#define PLUGIN_AUDIO_DRIVER_H

#include <fstream>

#ifdef DEBUG
#include <iostream>
#endif

extern "C" {
#define ALSA_PCM_NEW_HW_PARAMS_API
#include <alsa/asoundlib.h>

#include <sys/ioctl.h>
#include <linux/soundcard.h>
#include <pthread.h>
}

typedef enum SOUND_DRIVER
{
	NONE	 = 0x00,
	ALSA_LIB = 0x01,
	ALSA_OSS = 0x02,
	DISK_WRITER = 0x03
};

extern char devices[256][3];
extern short *audio_bufs[2];
extern uint n_bytes;
extern int cur_buf;
extern pthread_t snd_thread;
extern void* ( *snd_thread_func )( void * );

class Plugin_Audio_Driver
{
public:
	Plugin_Audio_Driver();
	~Plugin_Audio_Driver();

	void initDriver( bool = false );
	void configureDriver( bool = false );

	void prepare();
	void pause();

	void setRate( int );
	void setDriver( SOUND_DRIVER );
	void setSync( bool );
	void setQuery( bool );
	void setVolumes( int, int );
	void setChannels( int );
	void setBufferTime( int );
	void setBufferSize( int );
	void setPeriodSize( int );

	void setDefaults();

	void getHardware( int & );
	const bool getDriver( SOUND_DRIVER ) const ;
	const SOUND_DRIVER getDriver() const ;
	const bool getSync() const;
	const bool getQuery() const;
	const uint getVolumeLeft() const;
	const uint getVolumeRight() const;
	const uint getVolumeMax() const;
	const uint getChannels() const;
	const uint getBufferTime() const;
	const uint getBufferSize() const;
	const uint getPeriodSize() const;

private:
	static void* alsa_thread_method1( void * );
	static void* alsa_thread_method2( void * );
	static void* oss_thread( void * );
	static void* disk_thread( void * );

	void allocBuffers();
	void deallocBuffers();

	void shutdownDriver();
	bool initAlsa();
	bool initOSS();
	bool initDiskWriter();

	bool drv_init[4];
	SOUND_DRIVER snd_drv;
	snd_pcm_hw_params_t *hw_params;
	snd_pcm_sw_params_t *sw_params;
	snd_mixer_t *mixer;
	snd_mixer_elem_t *mixer_elem;

	uint vol_left, vol_right, vol_max;
	uint n_channels, rate;
	int xfer, open_mode;
	snd_pcm_uframes_t buffer_size;
	snd_pcm_uframes_t period_size;

	bool sync_fix, query_hardware;
};

#endif //PLUGIN_AUDIO_DRIVER_H
