/***************************************************************************
                          gtk1_gui.cpp  -  description
                             -------------------
    begin                : Sat Oct 11 2003
    copyright            : (C) 2003 by Guilherme Fortunato
    email                : guilherme@linksat.com.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "gui.h"

#define ESC_KEY 65307
#define ENTER_KEY 65293

extern "C" {
	#include <gtk/gtk.h>
}

class GTK1_Interface
	: public Plugin_Graphical_Interface
{
public:
	GTK1_Interface( Plugin_Audio_Driver * );
	virtual ~GTK1_Interface();
	virtual void Destroy();
	virtual void Show( DIALOG_TYPE );
	virtual void setAudioParameters( bool = false );
	virtual void getAudioParameters();

	friend void begin_testing();
	friend bool key_press_event_func( GtkWidget *, GdkEventKey * );
	friend void loadDefaultValues();
	friend void loadConfigurationFromFile();
	friend void saveConfigurationToFile();
	friend void ok_pressed();
	friend void cancel_config();

private:
	struct _config_dlg_struct *config_dlg;
	struct _about_dlg_struct *about_dlg;

	bool configured[3];
	void Configure( DIALOG_TYPE );

	void setupConfigDialog();
	void setupAboutDialog();
};

void CreateGUIObject( Plugin_Graphical_Interface **GUI, Plugin_Audio_Driver *drv )
{
	( ( GTK1_Interface * )( *GUI ) ) = new GTK1_Interface( drv );
}

void DeleteGUIObject( Plugin_Graphical_Interface **GUI )
{
	delete *GUI;
	*GUI = NULL;
}

static GTK1_Interface *gui;

static GList *lst_channels;
static void mkList()
{
	int i = 0;
	do {
		lst_channels = g_list_insert( lst_channels, ( void * )cboValues[i], i );
	} while ( cboValues[++i] != NULL );
}

struct _config_dlg_struct
{
	GtkWidget *Dialog;
	GtkWidget *inputBufferTime, *inputBufferSize, *inputPeriodSize, *inputDiskFile;
	GtkWidget *btnLoad, *btnSave, *btnDefault, *btnTest;
	GtkWidget *btnOK, *btnCancel;
	GtkWidget *hbox, *vbox;
	GtkWidget *cboChannels;
	GtkWidget *chkLoad, *chkQuery, *chkSyncFix;
	GtkWidget *frame;
	GtkWidget *alsa_opt, *oss_opt, *disk_opt;
	GtkWidget *sliderLeft, *sliderRight;
	GtkObject *vol_adjusts_left, *vol_adjusts_right;
	GtkTooltips *tips;
	GSList *driver_grp;
};

struct _about_dlg_struct
{
	GtkWidget *Dialog;
	GtkWidget *btnOK;
};

GTK1_Interface::GTK1_Interface( Plugin_Audio_Driver *drv )
	: Plugin_Graphical_Interface( drv )
{
	gui = this;
}

GTK1_Interface::~GTK1_Interface()
{
	this->Destroy();
}

void GTK1_Interface::Show( DIALOG_TYPE type )
{
	if ( configured[static_cast<int>( type )] == false ) {
		Configure( type );
		configured[static_cast<int>( type )] = true;
	}
	switch ( type ) {
		case CONFIG:
		case TEST:
			backup();
			gtk_widget_show_all( config_dlg->Dialog );
			break;
		case ABOUT:
			gtk_widget_show_all( about_dlg->Dialog );
	}
}

void GTK1_Interface::Destroy()
{
	if ( configured[0] == true ) {
		gtk_widget_destroy( config_dlg->Dialog );
		delete config_dlg;
	}
	if ( configured[2] == true ) {
		gtk_widget_destroy( about_dlg->Dialog );
		delete about_dlg;
	}
}

void GTK1_Interface::Configure( DIALOG_TYPE type )
{
	switch ( type ) {
		case CONFIG:
		case TEST:
			setupConfigDialog();
			getAudioParameters();
			break;
		case ABOUT:
			setupAboutDialog();
	}
}

void GTK1_Interface::setAudioParameters( bool testing )
{
	setDriverFilename( gtk_entry_get_text( GTK_ENTRY( config_dlg->inputDiskFile ) ) );
	if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( config_dlg->disk_opt ) ) == true ) {
		if ( !testing ) {
			getAudioDriver()->setDriver( DISK_WRITER );
			return;
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( config_dlg->disk_opt ), false );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( config_dlg->alsa_opt ), true );
		}
	}

	if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( config_dlg->alsa_opt ) ) == true ) {
		getAudioDriver()->setDriver( ALSA_LIB );
	}
	else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( config_dlg->oss_opt ) ) == true ) {
		getAudioDriver()->setDriver( ALSA_OSS );
	}
	getAudioDriver()->setSync( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( config_dlg->chkSyncFix ) ) );
	getAudioDriver()->setQuery( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( config_dlg->chkQuery ) ) );
	getAudioDriver()->setVolumes( static_cast<uint>( GTK_ADJUSTMENT( config_dlg->vol_adjusts_left )->value ),
		static_cast<uint>( GTK_ADJUSTMENT( config_dlg->vol_adjusts_right )->value ) );
	getAudioDriver()->setChannels( static_cast<uint>( atoi( gtk_entry_get_text( GTK_ENTRY( GTK_COMBO( config_dlg->cboChannels )->entry ) ) ) ) );
	getAudioDriver()->setBufferTime( static_cast<uint>( atoi( gtk_entry_get_text( GTK_ENTRY( config_dlg->inputBufferTime ) ) ) ) );
	getAudioDriver()->setBufferSize( static_cast<uint>( atoi( gtk_entry_get_text( GTK_ENTRY( config_dlg->inputBufferSize ) ) ) ) );
	getAudioDriver()->setPeriodSize( static_cast<uint>( atoi( gtk_entry_get_text( GTK_ENTRY( config_dlg->inputPeriodSize ) ) ) ) );
	if ( !testing )
		setAutoLoad( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( config_dlg->chkLoad ) ) );
}

void GTK1_Interface::getAudioParameters()
{
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( config_dlg->alsa_opt ), ( getAudioDriver()->getDriver( ALSA_LIB ) ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( config_dlg->oss_opt ), ( getAudioDriver()->getDriver( ALSA_OSS ) ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( config_dlg->disk_opt ), ( getAudioDriver()->getDriver( DISK_WRITER ) ) );
	gtk_adjustment_set_value( GTK_ADJUSTMENT( config_dlg->vol_adjusts_left ), getAudioDriver()->getVolumeLeft() );
	gtk_adjustment_set_value( GTK_ADJUSTMENT( config_dlg->vol_adjusts_right ), getAudioDriver()->getVolumeRight() );
	gtk_entry_set_text( GTK_ENTRY( GTK_COMBO( config_dlg->cboChannels )->entry ), String::number( getAudioDriver()->getChannels() ).data() );
	gtk_entry_set_text( GTK_ENTRY( config_dlg->inputBufferTime ), String::number( getAudioDriver()->getBufferTime() ).data() ) ;
	gtk_entry_set_text( GTK_ENTRY( config_dlg->inputBufferSize ), String::number( getAudioDriver()->getBufferSize() ).data() );
	gtk_entry_set_text( GTK_ENTRY( config_dlg->inputPeriodSize ), String::number( getAudioDriver()->getPeriodSize() ).data() );
	gtk_entry_set_text( GTK_ENTRY( config_dlg->inputDiskFile ), getDriverFilename() );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( config_dlg->chkLoad ), getAutoLoad() );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( config_dlg->chkQuery ), getAudioDriver()->getQuery() );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( config_dlg->chkSyncFix ), getAudioDriver()->getSync() );
}

void GTK1_Interface::setupConfigDialog()
{
	config_dlg = new( _config_dlg_struct );
	config_dlg->Dialog = gtk_dialog_new();
	gtk_window_set_title( GTK_WINDOW( config_dlg->Dialog ), CONF_DLG_CAPTION );
	gtk_window_set_modal( GTK_WINDOW( config_dlg->Dialog ), true );
	gtk_window_set_position( GTK_WINDOW( config_dlg->Dialog ), GTK_WIN_POS_CENTER );
	gtk_signal_connect( GTK_OBJECT( config_dlg->Dialog ), "key_press_event", GTK_SIGNAL_FUNC( key_press_event_func ), NULL );

	config_dlg->frame = gtk_frame_new( LBL_DEVICE_LIST );
	config_dlg->vbox = gtk_vbox_new( true, 5 );
	int var_int = 0;
	getAudioDriver()->getHardware( var_int );
	if ( var_int ) {
		while ( --var_int >= 0 ) {
			gtk_box_pack_start( GTK_BOX( config_dlg->vbox ), gtk_label_new( devices[2 - var_int] ), false, false, 0 );
			gtk_container_add( GTK_CONTAINER( config_dlg->frame ), config_dlg->vbox );
		}
	}
	else
		gtk_container_add( GTK_CONTAINER( config_dlg->frame ), gtk_label_new( LBL_NO_DEVICE ) );

	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( config_dlg->Dialog )->vbox ), config_dlg->frame );
	config_dlg->tips = gtk_tooltips_new();

	config_dlg->alsa_opt = gtk_radio_button_new_with_label( NULL, LBL_ALSA );
	config_dlg->driver_grp = gtk_radio_button_group( GTK_RADIO_BUTTON( config_dlg->alsa_opt ) );
	config_dlg->oss_opt = gtk_radio_button_new_with_label ( config_dlg->driver_grp, LBL_OSS );
	config_dlg->disk_opt = gtk_radio_button_new_with_label(
		gtk_radio_button_group( GTK_RADIO_BUTTON( config_dlg->oss_opt ) ), LBL_DISK );

	config_dlg->hbox = gtk_hbox_new( false, 5 );
	config_dlg->inputDiskFile = gtk_entry_new();
	gtk_entry_set_editable( GTK_ENTRY( config_dlg->inputDiskFile ), true );
	gtk_container_add( GTK_CONTAINER( config_dlg->hbox ), config_dlg->disk_opt );
	gtk_container_add( GTK_CONTAINER( config_dlg->hbox ), config_dlg->inputDiskFile );

	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( config_dlg->Dialog )->vbox ), config_dlg->alsa_opt );
	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( config_dlg->Dialog )->vbox ), config_dlg->oss_opt );
	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( config_dlg->Dialog )->vbox ), config_dlg->hbox );

	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->alsa_opt, ALSA_TIP, NULL );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->oss_opt, OSS_TIP, NULL );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->disk_opt, DISK_TIP, NULL );

    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( config_dlg->Dialog )->vbox ), gtk_hseparator_new(), false, true, 10 );

	config_dlg->vol_adjusts_left = gtk_adjustment_new( ( long )getAudioDriver()->getVolumeLeft(), 0L, 101L, 1, 1, 1 );
	config_dlg->vol_adjusts_right = gtk_adjustment_new( ( long )getAudioDriver()->getVolumeRight(), 0L, 101L, 1, 1, 1 );
	config_dlg->sliderLeft = gtk_vscale_new( GTK_ADJUSTMENT( config_dlg->vol_adjusts_left ) );
	config_dlg->sliderRight = gtk_vscale_new( GTK_ADJUSTMENT( config_dlg->vol_adjusts_right ) );
	gtk_scale_set_value_pos( GTK_SCALE( config_dlg->sliderLeft ), GTK_POS_BOTTOM );
	gtk_scale_set_value_pos( GTK_SCALE( config_dlg->sliderRight ), GTK_POS_BOTTOM );
	gtk_scale_set_draw_value( GTK_SCALE( config_dlg->sliderLeft ), true );
	gtk_scale_set_draw_value( GTK_SCALE( config_dlg->sliderRight ), true );
	gtk_range_set_update_policy( GTK_RANGE( config_dlg->sliderLeft ), GTK_UPDATE_DISCONTINUOUS );
	gtk_range_set_update_policy( GTK_RANGE( config_dlg->sliderRight ), GTK_UPDATE_DISCONTINUOUS );
	gtk_scale_set_digits( GTK_SCALE( config_dlg->sliderLeft ), false );
	gtk_scale_set_digits( GTK_SCALE( config_dlg->sliderRight ), false );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->sliderLeft, VOL_TIP, NULL );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->sliderRight, VOL_TIP, NULL );

	GtkWidget *vbox2 = gtk_vbox_new( false, 5 );
	gtk_box_pack_start( GTK_BOX( vbox2 ), gtk_label_new( "Left vol." ), false, false, 0 );
	gtk_box_pack_start( GTK_BOX( vbox2 ), config_dlg->sliderLeft, false, false, 0 );

	GtkWidget *hbox2 = gtk_hbox_new( false, 5 );
	gtk_box_pack_start( GTK_BOX( hbox2 ), vbox2, false, false, 0 );

	vbox2 = gtk_vbox_new( false, 5 );
	gtk_box_pack_start( GTK_BOX( vbox2 ), gtk_label_new( "Right vol." ), false, false, 0 );
	gtk_box_pack_start( GTK_BOX( vbox2 ), config_dlg->sliderRight, false, false, 0 );
	gtk_box_pack_start( GTK_BOX( hbox2 ), vbox2, false, false, 0 );

	config_dlg->vbox = gtk_vbox_new( false, 5 );
	config_dlg->cboChannels = gtk_combo_new();
	gtk_tooltips_set_tip( config_dlg->tips, GTK_COMBO( config_dlg->cboChannels )->entry, CHANNEL_TIP, NULL );
	gtk_combo_set_use_arrows_always( GTK_COMBO( config_dlg->cboChannels ), true );

	mkList();
	gtk_combo_set_popdown_strings( GTK_COMBO( config_dlg->cboChannels ), lst_channels );
	config_dlg->hbox = gtk_hbox_new( false, 5 );

	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), gtk_label_new( LBL_CHANNELS ), false, false, 0 );
	gtk_entry_set_editable( GTK_ENTRY( GTK_COMBO( config_dlg->cboChannels )->entry ), false );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), config_dlg->cboChannels, false, false, 0 );
	gtk_box_pack_start( GTK_BOX( config_dlg->vbox ), config_dlg->hbox, true, false, 0 );

	config_dlg->hbox = gtk_hbox_new( false, 5 );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), gtk_label_new( LBL_BUFFER_TIME ), false, false, 0 );
	config_dlg->inputBufferTime = gtk_entry_new_with_max_length( 7 );
	gtk_entry_set_editable( GTK_ENTRY( config_dlg->inputBufferTime ), true );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->inputBufferTime, TIME_TIP, NULL );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), config_dlg->inputBufferTime, false, false, 0 );
	gtk_box_pack_start( GTK_BOX( config_dlg->vbox ), config_dlg->hbox, true, false, 0 );

	config_dlg->hbox = gtk_hbox_new( false, 5 );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), gtk_label_new( LBL_BUFFER_SIZE ), false, false, 0 );
	config_dlg->inputBufferSize = gtk_entry_new_with_max_length( 5 );
	gtk_entry_set_editable( GTK_ENTRY( config_dlg->inputBufferSize ), true );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->inputBufferSize, SIZE_TIP, NULL );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), config_dlg->inputBufferSize, true, true, 0 );
	gtk_box_pack_start( GTK_BOX( config_dlg->vbox ), config_dlg->hbox, true, false, 0 );

	config_dlg->hbox = gtk_hbox_new( false, 5 );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), gtk_label_new( LBL_PERIOD_SIZE ), false, false, 0 );
	config_dlg->inputPeriodSize = gtk_entry_new_with_max_length( 4 );
	gtk_entry_set_editable( GTK_ENTRY( config_dlg->inputPeriodSize ), true );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->inputPeriodSize, PERIOD_TIP, NULL );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), config_dlg->inputPeriodSize, true, true, 0 );
	gtk_box_pack_start( GTK_BOX( config_dlg->vbox ), config_dlg->hbox, true, false, 0 );

	config_dlg->hbox = gtk_hbox_new( false, 5 );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), hbox2, false, false, 0 );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), config_dlg->vbox, true, true, 0 );
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( config_dlg->Dialog )->vbox ), config_dlg->hbox, true, false, 0 );

    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( config_dlg->Dialog )->vbox ), gtk_hseparator_new(), false, true, 10 );

	config_dlg->chkSyncFix = gtk_check_button_new_with_label( LBL_SYNC );
	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( config_dlg->Dialog )->vbox ), config_dlg->chkSyncFix );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->chkSyncFix, SYNC_TIP, NULL );

	config_dlg->chkQuery = gtk_check_button_new_with_label( LBL_QUERY );
	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( config_dlg->Dialog )->vbox ), config_dlg->chkQuery );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->chkQuery, QUERY_TIP, NULL );

	config_dlg->chkLoad = gtk_check_button_new_with_label( LBL_LOAD );
	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( config_dlg->Dialog )->vbox ), config_dlg->chkLoad );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->chkLoad, LOAD_TIP, NULL );

	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( config_dlg->Dialog )->vbox ), gtk_hseparator_new(), false, true, 10 );

	config_dlg->hbox = gtk_hbox_new( false, 5 );

	config_dlg->btnDefault = gtk_button_new_with_label( "Default" );
	gtk_signal_connect_object( GTK_OBJECT( config_dlg->btnDefault ), "clicked",
		GTK_SIGNAL_FUNC( loadDefaultValues ), GTK_OBJECT( config_dlg->Dialog ) );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->btnDefault, DEFAULT_TIP, NULL );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), config_dlg->btnDefault, true, true, 0 );

	config_dlg->btnTest = gtk_button_new_with_label( "Test" );
	gtk_signal_connect_object( GTK_OBJECT( config_dlg->btnTest ), "clicked",
		GTK_SIGNAL_FUNC( begin_testing ), GTK_OBJECT( config_dlg->Dialog ) );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->btnTest, TEST_TIP, NULL );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), config_dlg->btnTest, true, true, 0 );

	config_dlg->btnLoad = gtk_button_new_with_label( "Load" );
	gtk_signal_connect_object( GTK_OBJECT( config_dlg->btnLoad ), "clicked",
		GTK_SIGNAL_FUNC( loadConfigurationFromFile ), GTK_OBJECT( config_dlg->Dialog ) );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->btnLoad, LOAD_TIP_BTN, NULL );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), config_dlg->btnLoad, true, true, 0 );

	config_dlg->btnSave = gtk_button_new_with_label( "Save" );
	gtk_signal_connect_object( GTK_OBJECT( config_dlg->btnSave ), "clicked",
		GTK_SIGNAL_FUNC( saveConfigurationToFile ), GTK_OBJECT( config_dlg->Dialog ) );
	gtk_tooltips_set_tip( config_dlg->tips, config_dlg->btnSave, SAVE_TIP, NULL );
	gtk_box_pack_start( GTK_BOX( config_dlg->hbox ), config_dlg->btnSave, true, true, 0 );

	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( config_dlg->Dialog )->vbox ), config_dlg->hbox, true, false, 0 );

	config_dlg->btnOK = gtk_button_new_with_label( "OK" );
	gtk_signal_connect_object( GTK_OBJECT( config_dlg->btnOK ), "clicked",
		GTK_SIGNAL_FUNC( ok_pressed ), GTK_OBJECT( config_dlg->Dialog ) );

	config_dlg->btnCancel = gtk_button_new_with_label( "Cancel" );
	gtk_signal_connect_object( GTK_OBJECT( config_dlg->btnCancel ), "clicked",
		GTK_SIGNAL_FUNC( cancel_config ), GTK_OBJECT( config_dlg->Dialog ) );

	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( config_dlg->Dialog )->action_area ), config_dlg->btnOK );
	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( config_dlg->Dialog )->action_area ), config_dlg->btnCancel );
}

void GTK1_Interface::setupAboutDialog()
{
	about_dlg = new( _about_dlg_struct );
	about_dlg->Dialog = gtk_dialog_new();
	gtk_window_set_title( GTK_WINDOW( about_dlg->Dialog ), ABOUT_DLG_CAPTION );
	gtk_window_set_position( GTK_WINDOW( about_dlg->Dialog ), GTK_WIN_POS_CENTER );
	about_dlg->btnOK = gtk_button_new_with_label( "OK" );
	gtk_signal_connect_object( GTK_OBJECT( about_dlg->btnOK ), "clicked",
		GTK_SIGNAL_FUNC( gtk_widget_hide ), GTK_OBJECT( about_dlg->Dialog ) );
	char about_text[150];
	sprintf( about_text, "%s v%.1f %s", PLUGIN_NAME, VERSION, ABOUT_TEXT );
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( about_dlg->Dialog )->vbox ), gtk_label_new( about_text ), false, true, 5 );
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( about_dlg->Dialog )->vbox ), gtk_hseparator_new(), false, true, 5 );
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( about_dlg->Dialog )->vbox ), about_dlg->btnOK, false, false, 5 );
	gtk_window_set_modal( GTK_WINDOW( about_dlg->Dialog ), true );
}

bool key_press_event_func( GtkWidget *widget, GdkEventKey *event )
{
	switch ( event->keyval ) {
		case ENTER_KEY:
			ok_pressed();
			break;
		case ESC_KEY:
			cancel_config();
	}
	return true;
}

void begin_testing()
{
	if ( !gui->isPlaying() )
		gui->setAudioParameters( true );
	gui->playTestFile();
}

void loadDefaultValues()
{
	gui->loadDefaults();
}

void loadConfigurationFromFile()
{
	gui->LoadConfiguration();
	gui->getAudioParameters();
}

void saveConfigurationToFile()
{
	gui->setAudioParameters();
	gui->SaveConfiguration();
}

void ok_pressed()
{
	gtk_widget_hide( gui->config_dlg->Dialog );
	gui->cancelConfig( true );
	gui->setAudioParameters();
}

void cancel_config()
{
	gtk_widget_hide( gui->config_dlg->Dialog );
	gui->cancelConfig();
}
