/***************************************************************************
                          gui.cpp  -  description
                             -------------------
    begin                : Sun Oct 19 2003
    copyright            : (C) 2003 by Guilherme Fortunato
    email                : guilherme@linksat.com.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "gui.h"

char disk_out_file[1024] = { "UNDEFINED" };

static char cfg_file_name[1024];
const char *cboValues[] = {
	"1", "2", "4", "5", "6", NULL
};

static int var_int;
static int backup_vals[10];

static const char *sections[9] = {
	"DRIVER", "SYNC", "QUERY", "TEST_FILE",
	"VOLUME", "CHANNELS", "BUFFERTIME", "BUFFERSIZE", "PERIODSIZE"
};


bool Plugin_Graphical_Interface::isFileOK( const char *file )
{
	if ( strlen( file ) ) {
		if ( access( file, F_OK ) != 0 ) {
			creat( file, 0666 );
			return false;
		}
		else {
			struct stat check_size;
			stat( file, &check_size );
			return ( check_size.st_size >= 320000 );
		}
	}
	return false;
}

static void getConfigFile()
{
	var_int = readlink( "/proc/self/exe", cfg_file_name, 1023 );
	cfg_file_name[var_int] = '\0';
	while( cfg_file_name[--var_int] != '/' );
	cfg_file_name[var_int] = '\0';
	strcat( cfg_file_name, "/plugins/" );
	strcat( cfg_file_name, CONFIG_FILE );
	if ( access( cfg_file_name, F_OK ) == 0 ) {
		if ( access( cfg_file_name, W_OK ) < 0 ) {
			memset( cfg_file_name, 0, var_int + 10 );
			sprintf( cfg_file_name, "%s/.%s", getenv( "HOME" ), CONFIG_FILE );
		}
	}
}

static FILE *test_file;
static bool playing;
static void *test_thread_func( void *emp )
{
	while ( playing ) {
		if ( !feof( test_file ) ) {
			fread( *( audio_bufs + 1 - cur_buf ), n_bytes, 1, test_file );
			pthread_join( snd_thread, NULL );
			cur_buf = 1 - cur_buf;
			pthread_create( &snd_thread, NULL, snd_thread_func, NULL );
		}
		else {
			rewind( test_file );
		}
	}
	return NULL;
}

Plugin_Graphical_Interface::Plugin_Graphical_Interface( Plugin_Audio_Driver *drv )
{
	audio_drv = drv;
	playing = false;
	getConfigFile();
	LoadConfiguration();
}

Plugin_Audio_Driver *Plugin_Graphical_Interface::getAudioDriver() const
{
	return audio_drv;
}

void Plugin_Graphical_Interface::setDriverFilename( const char *name )
{
	strncpy( disk_out_file, name, strlen( name ) );
}

const char *Plugin_Graphical_Interface::getDriverFilename()
{
	return ( const char * )disk_out_file;
}

const bool Plugin_Graphical_Interface::getAutoLoad() const
{
	return auto_load;
}

void Plugin_Graphical_Interface::setAutoLoad( bool load )
{
	if ( ( auto_load = load ) )
		SaveConfiguration();
}

void Plugin_Graphical_Interface::LoadConfiguration()
{
	static bool configOK = false;
	if ( !configOK )
		getAudioDriver()->setDefaults();

	if ( access( cfg_file_name, F_OK ) < 0 ) return;
		cfg_file.open( cfg_file_name, std::ios::in | std::ios::ate );
	if ( cfg_file.fail() ) return;
		if ( ( var_int = cfg_file.tellg() ) == 0 ) return;
	file_content.Resize( var_int );
	cfg_file.seekg( 0 );
	cfg_file.read( ( char * )file_content.data(), var_int );
	cfg_file.close();

	int equal_char, new_line_char;
	equal_char = file_content.find( '=', 0 );
	new_line_char = file_content.find( '\n', equal_char + 1 );
	if ( file_content.mid( equal_char + 1, new_line_char - equal_char - 1 ).compare( "NO" ) == 0 ) {
		auto_load = false;
		configOK = true;
		return;
	}
	new_line_char = file_content.find( '{', equal_char + 1 ) + 1;
	String field_name;
	while ( true ) {
		equal_char = file_content.find( '=', new_line_char + 1 );
		if ( equal_char != -1 ) {
			field_name = file_content.mid( new_line_char + 1, equal_char - new_line_char - 1 );
			new_line_char = file_content.find( '\n', equal_char + 1 );
			if ( new_line_char != -1 )
				field_value = file_content.mid( equal_char + 1, new_line_char - equal_char - 1 );
		}
		else break;
		field_name.trimWhiteSpaces();
		field_value.trimWhiteSpaces();
		var_int = 0;
		do {
			if ( field_name.compare( *( sections + var_int ) ) == 0 ) break;
		} while ( ++var_int < 9 );

		switch ( var_int ) {
			case 0:
				if ( field_value.compare( "ALSA_LIB" ) == 0 ) {
					getAudioDriver()->setDriver( ALSA_LIB );
				}
				else if ( field_value.compare( "ALSA_OSS" ) == 0 ) {
					getAudioDriver()->setDriver( ALSA_OSS );
				}
				else if ( field_value.compare( "DISK_WRITER" ) == 0 )
					getAudioDriver()->setDriver(  DISK_WRITER );
				break;
			case 1:
				getAudioDriver()->setSync( field_value.compare( "NO_FORCE" ) == 0 ? false : true );
				break;
			case 2:
				getAudioDriver()->setQuery( field_value.compare( "NO" ) == 0 ? false : true );
				break;
			case 3:
				strncpy( disk_out_file, field_value.mid( 1, field_value.length() - 2 ).data(), field_value.length() - 2 );
				break;
			case 4:
				var_int = field_value.find( ',' );
				getAudioDriver()->setVolumes( field_value.left( var_int ).toInt(),
			 	field_value.mid( var_int + 1, field_value.length() - var_int ).toInt() );
				break;
			case 5:
				getAudioDriver()->setChannels( field_value.toInt() );
				break;
			case 6:
				getAudioDriver()->setBufferTime( field_value.toInt() );
				break;
			case 7:
				getAudioDriver()->setBufferSize( field_value.toInt() );
				break;
			case 8:
				getAudioDriver()->setPeriodSize( field_value.toInt() );
		}
	}
	auto_load = configOK = true;
}

void Plugin_Graphical_Interface::SaveConfiguration()
{
	cfg_file.open( cfg_file_name, std::ios::out | std::ios::trunc );
	if ( cfg_file.fail() ) return;
	file_content = String( "AUTO_LOAD=%1\nProfile[Default] {\n" ).arg( auto_load == true ? "YES" : "NO" );
	if ( getAudioDriver()->getDriver( ALSA_LIB ) ) field_value = "=ALSA_LIB\n";
	else if ( getAudioDriver()->getDriver( ALSA_OSS ) ) field_value = "=ALSA_OSS\n";
	else if ( getAudioDriver()->getDriver( DISK_WRITER ) ) field_value = "=DISK_WRITER\n";
	else field_value = "=NONE\n";
	file_content.append( sections[0] + field_value );
	field_value = getAudioDriver()->getSync() == true ? "=FORCE\n" : "=NO_FORCE\n";
	file_content.append( sections[1] + field_value );
	field_value = getAudioDriver()->getQuery() == true ? "=YES\n" : "=NO\n";
	file_content.append( sections[2] + field_value );
	field_value = String( "=\"%1\"\n" ).arg( ( const char * )disk_out_file );
	file_content.append( sections[3] + field_value );
	field_value = String( "=%1,%2\n" ).arg( getAudioDriver()->getVolumeLeft() ).arg( getAudioDriver()->getVolumeRight() );
	file_content.append( sections[4] + field_value );
	field_value = String( "=%1\n" ).arg( getAudioDriver()->getChannels() );
	file_content.append( sections[5] + field_value );
	field_value = String( "=%1\n" ).arg( getAudioDriver()->getBufferTime() );
	file_content.append( sections[6] + field_value );
	field_value = String( "=%1\n" ).arg( getAudioDriver()->getBufferSize() );
	file_content.append( sections[7] + field_value );
	field_value = String( "=%1\n}" ).arg( getAudioDriver()->getPeriodSize() );
	file_content.append( sections[8] + field_value );
	cfg_file.write( ( char * )( file_content.data() ), file_content.length() );
	cfg_file.close();
}

void Plugin_Graphical_Interface::loadDefaults()
{
	audio_drv->setDefaults();
	getAudioParameters();
}

void Plugin_Graphical_Interface::backup()
{
	*backup_vals = audio_drv->getDriver();
	*( backup_vals + 1 ) = audio_drv->getSync();
	*( backup_vals + 2 ) = audio_drv->getQuery();
	*( backup_vals + 3 ) = audio_drv->getVolumeLeft();
	*( backup_vals + 4 ) = audio_drv->getVolumeRight();
	*( backup_vals + 5 ) = audio_drv->getChannels();
	*( backup_vals + 6 ) = audio_drv->getBufferTime();
	*( backup_vals + 7 ) = audio_drv->getBufferSize();
	*( backup_vals + 8 ) = audio_drv->getPeriodSize();
	*( backup_vals + 9 ) = getAutoLoad();
}

void Plugin_Graphical_Interface::cancelConfig( bool leave_here )
{
	if ( playing ) {
		playing = false;
		pthread_join( test_thread, NULL ); // Maybe remove this
		audio_drv->pause();
		fclose( test_file );
	}
	if ( leave_here ) return;
	audio_drv->setDriver( static_cast<SOUND_DRIVER>( *backup_vals ) );
	audio_drv->setSync( *( backup_vals + 1 ) );
	audio_drv->setQuery( *( backup_vals + 2 ) );
	audio_drv->setVolumes( *( backup_vals + 3 ), *( backup_vals + 4 ) );
	audio_drv->setChannels( *( backup_vals + 5 ) );
	audio_drv->setBufferTime( *( backup_vals + 6 ) );
	audio_drv->setBufferSize( *( backup_vals + 7 ) );
	audio_drv->setPeriodSize( *( backup_vals + 8 ) );
	auto_load = *( backup_vals + 9 );
}

const bool Plugin_Graphical_Interface::isPlaying() const
{
	return playing;
}

bool Plugin_Graphical_Interface::playTestFile()
{
	if ( !playing ) {
		if ( !isFileOK( getDriverFilename() ) ) return false;
		test_file = fopen( getDriverFilename(), "rb" );
		if ( test_file < 0 ) return false;
		audio_drv->initDriver( playing = true );
		audio_drv->configureDriver();
		audio_drv->prepare();
		n_bytes = 2200;
		pthread_create( &test_thread, NULL, test_thread_func, NULL );
	}
	else
		cancelConfig( true );
	return true;
}
