/***************************************************************************
                          gui.h  -  description
                             -------------------
    begin                : Sat Oct 11 2003
    copyright            : (C) 2003 by Guilherme Fortunato
    email                : guilherme@linksat.com.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _PLUGIN_GRAPHICAL_INTERFACE_H_
#define _PLUGIN_GRAPHICAL_INTERFACE_H_

#define ALSA_TIP "This is the default driver. It is meant to work better, that is, smoother and without too much increase in processor usage"
#define OSS_TIP "This option is included for purposes of compatibility and comparison against ALSA. It should work even without the ALSA system running( but you need ALSA to compile this plugin, though ). If you are using ALSA, make sure the kernel module snd-pcm-oss is loaded"
#define DISK_TIP "Use the option to record sound samples from any game to the hard disk. Later you can use the recorded file to test some configuration"

#define VOL_TIP "With ALSA library it will work fine up to 4 four channels( untested with more ); front and rear sides share the same volume. For OSS, both channels will have the same volume( the left side's )."
#define CHANNEL_TIP "All cards support 2 channels. If you choose 4 channels it means your card will have to support Surround Sound. 5 channels, Dolby 5.0. 6 channels, Dolby 5.1. Currently, OSS supports only 2 channels"
#define TIME_TIP "Try a value within the range: 1000000 to 3000000, to see what is best"
#define SIZE_TIP "Larger values will, possibly, make the emulator pause to wait the audio card to finish processing the whole buffer. Lower values would produce noise"
#define PERIOD_TIP "The deault value is a sixteenth of that of buffer size. Higher values will increase latency."
#define SYNC_TIP "Only useful for ALSA. Attempts to maximize system resources while providing fine synchronization. Ironically, if you test this option you will notice synchronization is broken"
#define QUERY_TIP "Check this if you want the plugin to probe the hardware for would-be optimal values."
#define LOAD_TIP "Check this if you want to have this configuration used everytime you start a game"
#define DEFAULT_TIP "Use the plugin's default values."
#define TEST_TIP "If you have recorded a file with sound samples from any game, try here to test your configuration. Bear in mind that as far as speed and synchronization go, the real test is to actually play a game"
#define SAVE_TIP "Store current information to the file mupen64_alsa.conf"
#define LOAD_TIP_BTN "Load stored information from the file mupen64_alsa.conf"

#define ABOUT_TEXT "by Guilherme\nContact: guilherme@linksat.com.br\n"\
	"Advanced Linux Sound Architecture home page: http://www.alsa-project.org/"

#define PLUGIN_NAME "Mupen64 ALSA2 Sound Plugin"
#define CONFIG_FILE "mupen64_alsa.conf"
#define CONF_DLG_CAPTION "Mupen 64: Audio Plugin Configuration"
#define FILE_DLG_CAPTION "Select a file ( existing or not ) for output"
#define ABOUT_DLG_CAPTION "About audio plugin"

#define LBL_DEVICE_LIST "Hardware device list: "
#define LBL_NO_DEVICE "No hardware found."
#define LBL_ALSA "Use ALSA Library"
#define LBL_OSS "Use ALSA's OSS Compatibility Layer"
#define LBL_DISK "Use Disk Writer plugin for testing"
#define LBL_CHANNELS "Number of channels: "
#define LBL_BUFFER_TIME "Hardware buffer time:"
#define LBL_BUFFER_SIZE "Soft and Hardware buffer size:"
#define LBL_PERIOD_SIZE "Soft and Hardware period size:"
#define LBL_SYNC "Better sound synchronization"
#define LBL_QUERY "Query hardware for values"
#define LBL_LOAD "Load configuration from file when starting up"

#include "string_class.h"

#include "audio.h"

#include <fstream>

extern "C" {
	#include <pthread.h>
}

extern const char *cboValues[];

typedef enum DIALOG_TYPE {
	CONFIG =	0x00,
	TEST =		0x01,
	ABOUT  =	0x02,
};

class Plugin_Graphical_Interface
{

public:

	Plugin_Graphical_Interface( Plugin_Audio_Driver * );
	virtual ~Plugin_Graphical_Interface(){}
	virtual void Destroy() {}
	virtual void Show( DIALOG_TYPE ) {}
	virtual void setAudioParameters( bool = false ) {}
	virtual void getAudioParameters() {}

	Plugin_Audio_Driver *getAudioDriver() const;

	void cancelConfig( bool = false );
	void backup();

	bool playTestFile();
	static bool isFileOK( const char * );

	void loadDefaults();
	void LoadConfiguration();
	void SaveConfiguration();

	const bool getAutoLoad() const;
	void setAutoLoad( bool );

	const bool isPlaying() const;

	void setDriverFilename( const char * );
	const char *getDriverFilename();

private:
	Plugin_Audio_Driver *audio_drv;
	pthread_t test_thread;
	std::fstream cfg_file;
	String file_content, field_value;
	bool auto_load, recording;
};

extern void CreateGUIObject( Plugin_Graphical_Interface **, Plugin_Audio_Driver * );
extern void DeleteGUIObject( Plugin_Graphical_Interface ** );

#endif //_PLUGIN_GRAPHICAL_INTERFACE_H_
