/***************************************************************************
                          snd_interface.cpp  -  description
                             -------------------
    begin                : Sat Oct 11 2003
    copyright            : (C) 2003 by Guilherme Fortunato
    email                : guilherme@linksat.com.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "gui.h"
#include "audio.h"
#include "snd_interface.h"

extern "C" {
	#include <pthread.h>
	#include "Audio_#1.1.h"
}

static AUDIO_INFO AudioInfo;
static Plugin_Audio_Driver *AudioDrv;
static Plugin_Graphical_Interface *GUI;
static short *in, *out, *end;

static void
setup_interface()
{
	AudioDrv = new Plugin_Audio_Driver();
	CreateGUIObject( &GUI, AudioDrv );
}

EXPORT void CALL
RomClosed( void )
{
	AudioDrv->pause();
}

EXPORT void CALL
AiDacrateChanged( int SystemType )
{
	int rate = 32000;
	switch ( SystemType ) {
		case SYSTEM_NTSC:
			rate = 48681812 / ( *AudioInfo.AI_DACRATE_REG + 1 ); break;
		case SYSTEM_PAL:
			rate = 49656530 / ( *AudioInfo.AI_DACRATE_REG + 1 ); break;
		case SYSTEM_MPAL:
			rate = 48628316 / ( *AudioInfo.AI_DACRATE_REG + 1 );
	}
	AudioDrv->setRate( rate );
}

EXPORT void CALL
AiLenChanged( void )
{
	in = ( short * )( AudioInfo.RDRAM + ( *AudioInfo.AI_DRAM_ADDR_REG & 0xFFFFFF ) );
	if ( !( *( in + 1 ) ) ) return;
	n_bytes = *AudioInfo.AI_LEN_REG;
	end = &in[n_bytes>>1];
	out = audio_bufs[1 - cur_buf];
	do {
		*out++ = *in++;
		*out++ = *in++;
		*out++ = *in++;
		*out++ = *in++;
		*out++ = *in++;
		*out++ = *in++;
		*out++ = *in++;
		*out++ = *in++;
	} while ( in < end );
	pthread_join( snd_thread, NULL );
	cur_buf = 1 - cur_buf;
	pthread_create( &snd_thread, NULL, snd_thread_func, NULL );
}

EXPORT BOOL CALL
InitiateAudio( AUDIO_INFO Audio_Info )
{
	AudioInfo = Audio_Info;
	AudioDrv->initDriver();
	AudioDrv->configureDriver();
	AudioDrv->prepare();
	return true;
}

EXPORT void CALL
DllClose()
{
	DeleteGUIObject( &GUI );
	delete AudioDrv;
}

EXPORT void CALL
GetDllInfo( PLUGIN_INFO *PluginInfo )
{
	setup_interface();
	PluginInfo->Version = ( WORD )VERSION;
	PluginInfo->Type = PLUGIN_TYPE_AUDIO;
	sprintf( PluginInfo->Name, PLUGIN_NAME );
	PluginInfo->NormalMemory  = true;
	PluginInfo->MemoryBswaped = false;
}

EXPORT void CALL
DllTest ( HWND hParent )
{
	GUI->Show( TEST );
}

EXPORT void CALL
DllAbout( HWND hParent )
{
	GUI->Show( ABOUT );
}

EXPORT void CALL
DllConfig ( HWND hParent )
{
	GUI->Show( CONFIG );
}
