/***************************************************************************
                          string_class.cpp  -  description
                             -------------------
    begin                : Fri Jul 25 2003
    copyright            : (C) 2003 by Guilherme Fortunato
    email                : guilherme@linksat.com.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cstring>

extern "C" {
	#include <limits.h>
}

#include "string_class.h"

#define MIN( num1, num2 )	( ( num1 ) < ( num2 ) ? ( num1 ) : ( num2 ) )

static ushort space_chars[] = { 9, 10, 11, 12, 13, 32 };
static ushort number_chars[] = { 48, 49, 50, 51, 52, 53, 54, 55, 56, 57 };

static ushort letter_chars[] = {
						65, 66, 67, 68, 69, 70, 71, 72, 73, 74,
						75, 76, 77, 78, 79, 80, 81, 82, 83, 84,
						85, 86, 87, 88, 89, 90, 97, 98, 99, 100,
						101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
						111, 112, 113, 114, 115, 116, 117, 118, 119, 120,
						121, 122
					};

static ushort ext_letter_chars[] = {
								192, 193, 194, 195, 196, 197, 198, 199, 200, 201,
								202, 203, 204, 205, 206, 207, 208, 209, 210, 211,
								212, 213, 214, 216, 217, 218, 219, 220, 221, 224,
								225, 226, 227, 228, 229, 230, 231, 232, 233, 234,
								235, 236, 237, 238, 239, 240, 241, 242, 243, 244,
								245, 246, 248, 249, 250, 251, 252, 253, 255
							};

static ushort punct_chars[] = { 33, 44, 45, 46, 58, 59, 63, 191 };

static ushort symbol_chars[] = {
						34, 35, 36, 37, 38, 39, 40, 41, 42, 43,
						47, 60, 61, 62, 64, 91, 92, 93, 94, 95,
						96, 123, 124, 125, 126, 128, 161, 162, 163, 164,
						165, 166, 167, 168, 169, 170, 171, 172, 173, 174,
						175, 176, 177, 178, 179, 180, 181, 182, 183, 184,
						185, 186, 187, 188, 189, 190, 215, 222, 223, 247,
						254,
					};

static ushort no_printable_chars[] = {
						0, 1, 2, 3, 4, 5, 6, 7, 8, 14,
						15, 16, 17, 18, 19, 20, 21, 22, 23, 24,
						25, 26, 27, 28, 29, 30, 31, 127, 129, 130,
						131, 132, 133, 134, 135, 136, 137, 138, 139, 140,
						141, 142, 143, 144, 145, 146, 147, 148, 149, 150,
						151, 152, 153, 154, 155, 156, 157, 158, 159, 160
					};

static int cs_strcmp( const String &as, const String &bs )
{
	const char *a = as.data();
	const char *b = bs.data();
	int l = MIN( as.length(), bs.length() );
	if ( !l ) return 0;
	while ( l-- && *a == *b )
		a++, b++;
	if ( l == -1 )
		return ( as.length() - bs.length() );
	return static_cast<int>( a - b );
}

static int ci_strcmp( const String &as, const String &bs )
{
	const Char *a = as.Data();
	const Char *b = bs.Data();
	int l = MIN( as.length(), bs.length() );
	if ( !l ) return 0;
	while ( l-- && ( a->toLower() == b->toLower() ) ) {
		a++, b++;
	}
	if ( l == -1 )
		return ( as.length() - bs.length() );
	return static_cast<int>( a - b );
}

inline static int cs_strncmp( const Char *a, const Char *b, int l )
{
	while ( l-- && *a == *b )
		a++, b++;
	if ( l == -1 )
		return 0;
	return static_cast<int>( a - b );
}

inline static int ci_strncmp( const Char *a, const Char *b, int l )
{
	while ( l-- && a->toLower() == b->toLower() )
		a++, b++;
	if ( l == -1 ) return 0;
	return a->toLower().integerValue() - b->toLower().integerValue();
}

inline static bool ok_in_base( Char c, int base )
{
	if ( base <= 10 )
		return c.isNumber() && c.digitValue() < base;
	else
		return c.isNumber() || ( c >= 'a' && c < char( 'a' + base - 10 ) )
			|| ( c >= 'A' && c < char( 'A' + base - 10 ) );
}

/* An implementation of the Boyer-Moore search algorithm
 * Initializes the skiptable to know how far ahead we can skip on a wrong match */
static void bm_init_skiptable( const String &pattern, uint *skiptable, bool cs )
{
	int i = 0;
	register uint *st = skiptable;
	int l = pattern.length();
	int max = 0x100 / 8;
	while ( i++ < max ) {
		*( st++ ) = l;
		*( st++ ) = l;
		*( st++ ) = l;
		*( st++ ) = l;
		*( st++ ) = l;
		*( st++ ) = l;
		*( st++ ) = l;
		*( st++ ) = l;
    }
	//const Char *uc = &pattern[0];
	const Char *uc = pattern.Data();
	if ( cs ) {
		while( l-- ) {
			skiptable[static_cast<uint>( uc->integerValue() ) & 0xff] = l;
			uc++;
		}
    }
	else {
		while( l-- ) {
			uc->toLower();
	    	skiptable[static_cast<uint>( uc->integerValue() ) & 0xff] = l;
	    	uc++;
		}
	}
}

static int bm_find( const String &str, int index, const String &pattern, uint *skiptable, bool cs )
{
	const uint l = str.length();
	if ( pattern.isEmpty() )
		return index > static_cast<int>( l ) ? -1 : index;

	const Char *uc = str.Data();
	const Char *puc = pattern.Data();
	const uint pl = pattern.length();
	const uint pl_minus_one = pl - 1;

	register const Char *current = uc + index + pl_minus_one;
	const Char *end = uc + l;
	uint skip;
	if ( cs ) {
		while( current < end ) {
			skip = skiptable[static_cast<uint>( current->integerValue() ) & 0xff];
			if ( !skip ) { // possible match
				while( skip < pl ) {
					if ( *( current - skip ) != *( puc + ( pl_minus_one - skip ) ) )
						break;
		    		skip++;
				}
				if ( skip > pl_minus_one ) { // we have a match
					return ( current - uc ) - skip + 1;
				}
				/* in case we don't have a match we are a bit inefficient as we only skip by one
				 * when we have the non matching char in the string. */
				if ( skiptable[static_cast<uint>( ( current - skip )->integerValue() ) & 0xff] == pl )
					skip = pl - skip;
				else
					skip = 1;
			}
			current += skip;
		}
	}
	else {
		while( current < end ) {
			skip = skiptable[static_cast<uint>( current->toLower().integerValue() ) & 0xff ];
			if ( !skip ) { // possible match
				while( skip < pl ) {
					if ( ( current - skip )->toLower() != ( puc + pl_minus_one - skip )->toLower() )
						break;
					skip++;
				}
				if ( skip > pl_minus_one ) // we have a match
					return ( current - uc ) - skip + 1;
				if ( skiptable[static_cast<uint>( ( current - skip )->toLower().integerValue() & 0xff ) ] == pl )
					skip = pl - skip;
				else
					skip = 1;
			}
			current += skip;
		}
	}
	return -1; // not found
}

#define REHASH( a ) \
	if ( sl_minus_1 < sizeof(uint) * CHAR_BIT ) \
	hashHaystack -= (a) << sl_minus_1; \
	hashHaystack <<= 1


bool Char::belongs( Category name, uchar character ) const
{
	ushort *ptr_start, *ptr_end;
	switch( name ) {
		case CHR_SPACE:
			ptr_start = space_chars;
			ptr_end = &space_chars[5];
			break;
		case CHR_NUMBER:
			ptr_start = number_chars;
			ptr_end = &number_chars[9];
			break;
		case CHR_LETTER:
			ptr_start = letter_chars;
			ptr_end = &letter_chars[51];
			break;
		case CHR_EXT_LETTER:
			ptr_start = ext_letter_chars;
			ptr_end = &ext_letter_chars[58];
			break;
		case CHR_PUNCT:
			ptr_start = punct_chars;
			ptr_end = &punct_chars[7];
			break;
		case CHR_SYMBOL:
			ptr_start = symbol_chars;
			ptr_end = &symbol_chars[60];
			break;
		default:
			ptr_start = no_printable_chars;
			ptr_end = &no_printable_chars[59];
	}
	for ( ; ptr_start <= ptr_end; ptr_start++ ) {
		if ( static_cast<ushort>( character ) == *ptr_start ) return true;
	}
	return false;
}

inline String::String( uint len ) : alloc_chars( 0 )
{
	Resize( len );
}

void String::DeAlloc()
{
	free( static_cast<void *>( c_data ) );
	c_data = 0;
}

// This function does not alter n_chars;
void String::Resize( uint new_n_chars )
{
	if ( !new_n_chars ) new_n_chars++;
	if ( alloc_chars && ( new_n_chars >= alloc_chars ) ) {
		alloc_chars = new_n_chars + 10;
		c_data = static_cast<Char *>( realloc( c_data,  alloc_chars * sizeof( Char ) ) );
	}
	else if ( !alloc_chars ) {
		alloc_chars = new_n_chars + 10;
		c_data = static_cast<Char *>( calloc( alloc_chars, sizeof( Char ) ) );
	}
	*( c_data + new_n_chars ) = '\0';
	n_chars = new_n_chars;
}

/* Just 1-digit arguments. */
bool String::findArg( int &pos, int &len ) const
{
	char lowest = 0;
	register const Char *uc = &c_data[0];
	Char dig;

	for( uint i = 0; i < n_chars; i++, uc++ ) {
		if ( *uc == '%' && ( i + 1 < n_chars ) ) {
			dig = *( uc + 1 );
			if ( dig >= '0' && dig <= '9' ) {
				if ( !lowest || dig < lowest ) {
					lowest = dig();
					pos = i;
					len = 2;
				}
	    	}
		}
    }
    return lowest != 0;
}

String &String::insert( uint index, const char *s, uint len )
{
	if ( !len ) return *this;
	uint olen = n_chars;
	int nlen = len + n_chars;

/*	if ( s >= c_data && ( uint )( s - c_data ) < 30 ) { // Part of me - take a copy.
		Char *tmp = static_cast<Char *>( malloc( len * sizeof( Char ) ) );
		memcpy( tmp, s, len * sizeof( Char ) );
		insert( index, tmp, len);
		free( static_cast<void *>( tmp ) );
		return *this;
    }*/

	if ( index >= olen ) { // insert after end of string
		Resize( n_chars + index + nlen );
		int n = index - olen;
		Char *uc = &c_data[olen];
		while ( n-- )
			*uc++ = ' ';
		strncpy( ( char * )&c_data[index], s, len );
    }
	else { // normal insert
		Resize( nlen );
		memmove( &c_data[index + len], &c_data[index], ( olen - index ) );
		strncpy( ( char * )&c_data[index], s, len );
	}
	return *this;
}

String &String::remove( uint index, uint len )
{
	if ( index >= n_chars ) return *this;

	if ( index + len >= n_chars )
		Resize( index );
    else if ( len ) {
		memmove( ( c_data + index ), ( c_data + index + len ), ( n_chars - index - len ) * sizeof( Char ) );
		Resize( n_chars - len );
    }
    return *this;
}

String &String::replace( uint index, uint len, const char *s, uint slen )
{
	if ( len == slen && index + len <= n_chars ) { // Optimized common case: replace without size change
		memcpy( &c_data[index], s, len * sizeof( Char ) );
    }
	/*else if ( s >= c_data && ( uint )( s - c_data ) < 30 ) { // Part of me - take a copy.
		Char *tmp = static_cast<Char *>( malloc( slen * sizeof( Char ) ) );
		memcpy( tmp, s, slen * sizeof( Char ) );
		replace( index, len, tmp, slen );
		free( static_cast<void *>( tmp ) );
    }*/
	else {
		remove( index, len );
		insert( index, s, slen );
	}
	return *this;
}

inline Char &String::at( int pos )
{
	return *( c_data + pos );
}


Char Char::null = '\0';
String String::null = Char::null;

/*----------------------------------------STRING----------------------------------------*/

String::String( Char c ) : alloc_chars( 0 )
{
	Resize( 1 );
	*c_data = c;
}

String::String( char c ) : alloc_chars( 0 )
{
	Resize( 1 );
	*c_data = c;
}

String::String( const char *str ) : alloc_chars( 0 )
{
	int nbr_chars = strlen( str );
	Resize( nbr_chars );
	strncpy( ( char * )c_data, str, nbr_chars );
}

/* Constructs an implicitly shared copy of s. This is very fast
 * since it only involves incrementing a reference count.*/
String::String( const String &str ) : alloc_chars( 0 )
{
	Resize( str.length() );
	memcpy( c_data, str.c_data, ( n_chars + 1 ) * sizeof( Char ) );
}

bool String::isNull() const
{
	return ( c_data == NULL );
}

bool String::isEmpty() const
{
	return ( length() == 0 );
}

uint String::length() const
{
	return n_chars;
}

void String::truncate( uint where )
{
	if ( where < n_chars )
		Resize( where );
}

String &String::fill( Char c, int len )
{
	if ( len > static_cast<int>( length() ) ) return *this;

	if ( len < 0 )
		len = length();
	if ( !len ) {
		*this = "";
	}
	else {
		Char *nd = c_data;
		while ( len-- ) *nd++ = c;
    }
    return *this;
}

String &String::fill( char c, int len )
{
	return ( fill( Char( c ), len ) );
}

int String::find( char c, int index, bool cs ) const
{
	return find( Char( c ), index, cs);
}

int String::find( const char *str, int index, bool cs ) const
{
	return find( String( str ), index, cs );
}

int String::find( Char c, int index, bool cs ) const
{
	if ( index < 0 )
		index += length();
	if ( static_cast<uint>( index ) >= length() )
		return -1;

	register const Char *uc = &c_data[index];
	const Char *end = &c_data[length() - 1];

	if ( cs ) {
		while ( uc <= end && *uc != c )
			uc++;
	}
	else {
		c = c.toLower();
		while ( uc <= end && uc->toLower() != c )
			uc++;
	}
	if ( ( static_cast<uint>( uc - c_data ) ) >= length() ) return -1;
    return static_cast<int>( uc - c_data );
}

int String::find( const String &str, int index, bool cs ) const
{
	const uint l = length();
	if ( index < 0 ) index += l;

	const uint sl = str.length();
	if ( !sl ) return index;
	if ( sl + index > l ) return -1;

	/* we use the Boyer-Moore algorithm in cases where the overhead
	 * for the hash table should pay off, otherwise we use a simple hash function */
	if ( l > 500 && sl > 5 ) {
		uint skiptable[0x100];
		bm_init_skiptable( str, skiptable, cs );
		return bm_find( *this, index, str, skiptable, cs );
	}

	/* We use some hashing for efficiency's sake. Instead of comparing strings,
	 * we compare the hash value of str with that of a part of this String. Only
	 * if that matches, we call strncmp or strnicmp. */
	const Char *needle = str.c_data;
	const Char *haystack = c_data + index;
	const Char *end = c_data + ( l - sl );
	const uint sl_minus_1 = sl - 1;
	uint hashNeedle = 0, hashHaystack = 0, i = 0;

	if ( cs ) {
		for (  ; i < sl; ++i ) {
			hashNeedle = ( ( hashNeedle << 1 ) + needle->integerValue() );
			hashHaystack = ( ( hashHaystack << 1 ) + haystack->integerValue() );
			needle++;
			haystack++;
		}
		haystack = c_data + index; // Return pointer to original position
		needle = str.c_data;

		hashHaystack -= ( haystack + sl_minus_1 )->integerValue();

		while ( haystack <= end ) {
			hashHaystack += ( haystack + sl_minus_1 )->integerValue();
			if ( hashHaystack == hashNeedle && ci_strncmp( needle, haystack, sl ) == 0 )
			return haystack - c_data;
			REHASH( haystack->integerValue() );
			++haystack;
		}
    }
	else {
		for ( ; i < sl; ++i ) {
			hashNeedle = ( ( hashNeedle << 1 ) + needle->toLower().integerValue() );
			hashHaystack = ( ( hashHaystack << 1 ) + haystack->toLower().integerValue() );
			needle++;
			haystack++;
		}
		haystack = c_data + index; // Return pointer to original position
		needle = str.c_data;

		hashHaystack -= ( haystack + sl_minus_1 )->toLower().integerValue();
		while ( haystack <= end ) {
			hashHaystack += ( haystack + sl_minus_1 )->toLower().integerValue();
			if ( hashHaystack == hashNeedle && cs_strncmp( needle, haystack, sl ) == 0 )
				return haystack - c_data;
			REHASH( haystack->toLower().integerValue() );
			++haystack;
		}
    }
    return -1;
}

int String::findRev( char c, int index, bool cs) const
{
	return findRev( Char( c ), index, cs );
}

int String::findRev( const char *str, int index, bool cs ) const
{
	return findRev( String( str ), index, cs );
}

int String::findRev( Char c, int index, bool cs ) const
{
	if ( index < 0 )
		index += length();
	if ( ( uint )index >= length() )
		return -1;

	const Char *end = c_data;
	register const Char *uc = ( c_data + index );
	if ( cs ) {
		while ( uc >= end && *uc != c )
	    uc--;
	}
	else {
		while ( uc >= end && uc->toLower() != c.toLower() )
		uc--;
	}
    return ( uc - end );
}


int String::findRev( const String &str, int index, bool cs ) const
{
	const uint l = length();
	if ( index < 0 ) index += l;
	const uint sl = str.length();
	int delta = l - sl;
	if ( index < 0 || index > static_cast<int>( l ) || delta < 0 ) return -1;
	if ( index > delta ) index = delta;

	const uint sl_minus_1 = sl - 1;
	const Char *needle = str.c_data + sl_minus_1; // sl_minus_1: begin search after search string
	const Char *haystack = c_data + index + sl_minus_1;
	const Char *end = c_data;
	uint hashNeedle = 0, hashHaystack = 0, i = 0;

	if ( cs ) {
		for ( ; i < sl; ++i ) {
			hashNeedle = ( ( hashNeedle << 1 ) + needle->integerValue() );
			hashHaystack = ( ( hashHaystack << 1 ) + haystack->integerValue() );
			needle--;
			haystack--;
		}
		haystack = c_data + index; // Return pointer to original position
		needle = str.c_data;

		hashHaystack -= haystack->integerValue();
		while ( haystack >= end ) {
			hashHaystack += haystack->integerValue();
			if ( hashHaystack == hashNeedle && ci_strncmp( needle, haystack, sl ) == 0 )
				return haystack - c_data;
			--haystack;
			REHASH( ( haystack + sl )->integerValue() );
		}
	}
	else {
		for ( ; i < sl; ++i ) {
			hashNeedle = ( ( hashNeedle << 1 ) + needle->toLower().integerValue() );
			hashHaystack = ( ( hashHaystack << 1 ) + haystack->toLower().integerValue() );
			needle--;
			haystack--;
		}
		haystack = c_data + index; // Return pointer to original position
		needle = str.c_data;

		hashHaystack -= haystack->toLower().integerValue();
		while ( haystack >= end ) {
			hashHaystack += haystack->toLower().integerValue();
			if ( hashHaystack == hashNeedle && cs_strncmp( needle, haystack, sl ) == 0 )
				return haystack - c_data;
			--haystack;
			REHASH( ( haystack + sl )->toLower().integerValue() );
		}
	}
	return -1;
}

int String::contains( const char *str, bool cs ) const
{
	return contains( String( str ), cs );
}

int String::contains( char c, bool cs ) const
{
	return contains( Char( c ), cs );
}

/* Returns the number of times the character c occurs in the string. */
int String::contains( Char c, bool cs ) const
{
	if ( !c_data )
		return 0;

	int count = 0;
	const Char *uc = c_data;
	int n = length();

	if ( cs ) {
		while ( n-- )
	    	if ( *uc++ == c )
				count++;
    }
	else {
		c = c.toLower();
		while ( n-- ) {
			if ( uc->toLower() == c )
				count++;
			uc++;
		}
	}
	return count;
}

/* Returns the number of times the string str occurs in the string. */
int String::contains( const String &str, bool cs ) const
{
	if ( isNull() ) return 0;
	int count = 0;
	uint skiptable[0x100];
	bm_init_skiptable( str, skiptable, cs );
	int i = -1;
	// use boyer-moore for the ultimate speed experience
	while ( ( i = bm_find( *this, i + 1, str, skiptable, cs ) ) != -1 )
		count++;
	return count;
}

String &String::prepend( char c )
{
	return insert( 0, Char( c ) );
}

String &String::prepend( Char c )
{
	return insert( 0, c );
}

String &String::prepend( const String &s )
{
	return insert( 0, s );
}

String &String::prepend( const char *c )
{
	return insert( 0, String( c ) );
}

String &String::append( char c )
{
	return operator+=( c );
}

String &String::append( Char c )
{
	return operator+=( c );
}

String &String::append( const String &s )
{
	return operator+=( s );
}

String &String::append( const char *s )
{
	return operator+=( s );
}

String String::arg( int a, int fieldwidth, uint base ) const
{
	return arg( String::number( a, base ), fieldwidth );
}

String String::arg( uint a, int fieldwidth, uint base ) const
{
	return arg( String::number( a, base ), fieldwidth );
}

String String::arg( char a, int fieldwidth ) const
{
	String c( a );
	return arg( c, fieldwidth );
}

String String::arg( Char a, int fieldwidth ) const
{
	String c( a );
	return arg( c, fieldwidth );
}

String String::arg( const char *a, int fieldwidth ) const
{
	String c( a );
	return arg( c, fieldwidth );
}

/* This function will return a string that replaces the lowest numbered occurrence
 * of %1, %2, ..., %9 with a. The fieldwidth value specifies the minimum amount of space that
 * a is padded to. A positive value will produce right-aligned text, whereas a negative
 * value will produce left-aligned text. */
String String::arg( const String &a, int fieldwidth ) const
{
	int pos, len;
	String r = *this;

	if ( !findArg( pos, len ) ) {
		// Make sure the text at least appears SOMEWHERE
		r += ' ';
		pos = r.length();
		len = 0;
	}

	r.replace( pos, len, a );
	if ( fieldwidth != 0 ) {
		if ( fieldwidth < 0 )
			fieldwidth = -fieldwidth;
		String s;
		while ( ( uint )fieldwidth > a.length() ) {
			s += ' ';
			fieldwidth--;
		}
		r.insert( pos, s );
	}
	return r;
}

String String::left( uint len ) const
{
	if ( isEmpty() )
		return String();
    if ( len >= length() )
		return *this;
	String s( len );
	memcpy( s.c_data, c_data, len * sizeof( Char ) );
	return s;
}

String String::right( uint len ) const
{
	if ( isEmpty() )
		return String();
	if ( len >= length() )
		return *this;
	String s( len );
	memcpy( s.c_data, &c_data[length() - len], len * sizeof( Char ) );
	return s;
}

String String::mid( uint index, uint len ) const
{
	if ( isEmpty() || index >= length() )
		return String();
	if ( len > ( length() - index ) )
		len = length() - index;
	if ( index == 0 && len == length() )
		return *this;
	String s( len );
	memcpy( s.c_data, ( c_data + index ), len * sizeof( Char ) );
	return s;
}

String String::leftJustify( uint width, Char fill, bool truncate ) const
{
	String result;
	int padlen = width - length();
	if ( padlen > 0 ) {
		result.Resize( length() + padlen );
		if ( length() )
			memcpy( result.c_data, c_data, length() * sizeof( Char ) );
		Char *uc = ( result.c_data + length() );
		while ( padlen-- )
			*uc++ = fill;
    }
	else {
		if ( truncate )
			result = left( width );
		else
			result = *this;
	}
	return result;
}

String String::rightJustify( uint width, Char fill, bool truncate ) const
{
	String result;
	int padlen = width - length();
	if ( padlen > 0 ) {
		result.Resize( n_chars + padlen );
		Char *uc = result.c_data;
		while ( padlen-- )
			*uc++ = fill;
		if ( length() )
			memcpy( uc, c_data, length() * sizeof( Char ) );
	}
	else {
		if ( truncate )
			result = left( width );
		else
			result = *this;
    }
	return result;
}

String String::lower() const
{
	register Char *p = c_data;
	Char *end = ( c_data + length() - 1 );
	while ( p <= end ) {
		*p = p->toLower();
		p++;
    }
    return *this;
}


String String::upper() const
{
	register Char *p = c_data;
	Char *end = ( c_data + length() - 1 );
	while ( p <= end ) {
		*p = p->toUpper();
		p++;
    }
    return *this;
}

/* Returns a string that has whitespace removed from the start and the end. */
String String::stripWhiteSpace() const
{
	if ( isEmpty() )
		return *this;

	Char *s = c_data;
	if ( !s->isSpace() && !( s + length() - 1 )->isSpace() )
		return *this;

	Char *end = ( c_data + length() - 1 );
	// skip white space from start
	while ( s <= end && s->isSpace() )
		s++;

	if ( s <= end ) { // skip white space from end
		end = s;
		s = &c_data[length() - 1];
		while ( s > end && s->isSpace() )
			s--;
	}
	if ( s == end )
		return String( "" );

	uint len = s - end;
	String result( len );
	memcpy( result.c_data, end, len * sizeof( Char ) );
	return result;
}

String &String::trimWhiteSpaces()
{
	if ( isEmpty() )
		return *this;

	uint pos = 0;
	while ( ( c_data + pos )->isSpace() ) pos++;
	if ( pos )
		remove( 0, pos - 1 );
	pos = length() - 1;
	while ( ( c_data + pos )->isSpace() ) pos--;
	if ( pos < ( length() - 1 ) )
		remove( pos + 2, length() - pos );
	return *this;
}

String &String::insert( uint index, char c ) // insert char
{
	return insert( index, ( const char * )&c, 1 );
}

String &String::insert( uint index, Char c ) // insert char
{
	char chr = c();
	return insert( index, ( const char * )&chr, 1 );
}

String &String::insert( uint index, const char *str )
{
	return insert( index, str, strlen( str ) );
}

String &String::insert( uint index, const String &s )
{
	return insert( index, ( const char * )s.c_data, s.length() );
}

inline String &String::remove( char c )
{
	return ( remove( Char( c ) ) );
}

/* Removes every occurrence of the character \a c in the string. */
String &String::remove( Char c )
{
	int i = 0;
	while ( i < static_cast<int>( length() ) ) {
		if ( at( i ) == c )
			remove( i, 1 );
		else
			i++;
	}
	return *this;
}

inline String &String::remove( const char *str )
{
	return remove( String( str ) );
}

String &String::remove( const String &str )
{
	int index = 0;
	if ( !str.isEmpty() ) {
		while ( ( index = find( str, index ) ) != -1 )
			remove( index, str.length() );
	}
	return *this;
}

/* Replaces every occurrence of c1 with the char c2.*/
String &String::replace( Char c1, Char c2 )
{
	register Char *c = c_data;
	Char *end = &c_data[length() - 1];
	while ( c <= end ) {
		if ( *c == c1 ) *c = c2;
		c++;
	}
	return *this;
}

String &String::replace( uint index, uint len, const char *str )
{
	return replace( index, len, str, strlen( str ) );
}

/* Replaces len characters from the string with s, starting at
 * position index, and returns a reference to the string. */
String &String::replace( uint index, uint len, const String &s )
{
	return replace( index, len, ( const char * )s.c_data, s.length() );
}

String &String::replace( uint index, uint len, char c )
{
	return replace( index, len, ( const char * )&c, 1 );
}

String &String::replace( uint index, uint len, Char c )
{
	char chr = c();
	return replace( index, len, ( const char * )&chr, 1 );
}

String &String::replace( char c, const String &after )
{
	return replace( String( c ), after );
}

String &String::replace( Char c, const String &after )
{
	return replace( String( c ), after );
}

/* Replaces every occurrence of the string before in the string
 * with the string after. Returns a reference to the string. */
String &String::replace( const String &before, const String &after )
{
	if ( before == after || isNull() ) return *this;
	int index = 0;
	uint skiptable[0x100];
	bm_init_skiptable( before, skiptable, true );
	const int bl = before.length();
	const int al = after.length();

	if ( bl == al ) {
		if ( bl ) {
			const Char *auc = after.c_data;
			while( ( index = bm_find( *this, index, before, skiptable, true ) ) != -1 ) {
				memcpy( ( c_data + index ), auc, al * sizeof( Char ) );
				index += bl;
			}
		}
	}
	else if ( al < bl ) {
		const Char *auc = after.c_data;
		uint to = 0;
		uint movestart = 0;
		uint num = 0;
		int msize;
		while( ( index = bm_find( *this, index, before, skiptable, true ) ) != -1 ) {
			if ( num ) {
				msize = index - movestart;
				if ( msize ) {
					memmove( c_data + to, c_data + movestart, msize * sizeof( Char ) );
					to += msize;
				}
			}
			else {
				to = index;
			}
			if ( al ) {
				memcpy( c_data + to, auc, al * sizeof( Char ) );
				to += al;
			}
			index += bl;
			movestart = index;
			num++;
		}
		if ( num ) {
			msize = n_chars - movestart;
			if ( msize )
				memmove( c_data + to, c_data + movestart, msize * sizeof( Char ) );
			Resize( n_chars - num * ( bl - al ) );
		}
	}
	else { /* the most complex case. We don't want to loose performance by doing repeated copies and reallocs of the string. */
		int adjust;
		uint indices[4096];
		uint *idx_ptr;
		uint pos;
		uint newlen;
		int movestart, moveend, moveto;
		int insertstart;
		while( index != -1 ) {
			idx_ptr = indices;
			pos = 0;
			while( pos < 4095 ) {
				index = bm_find( *this, index, before, skiptable, true );
				if ( index == -1 ) break;
				*( idx_ptr + pos++ ) = index;
				index += bl;
				// avoid infinite loop
				if ( !bl ) index++;
			}
			if ( !pos ) break;
			// we have a table of replacement positions, use them for fast replacing
			adjust = pos * ( al - bl );
			// index has to be adjusted in case we get back into the loop above.
			if ( index != -1 )
				index += adjust;
			newlen = n_chars + adjust;
			moveend = n_chars;
			if ( newlen > n_chars )
				Resize( newlen );
			while( pos ) {
				pos--;
				movestart = *( idx_ptr + pos ) + bl;
				insertstart = *( idx_ptr + pos ) + pos * ( al - bl );
				moveto = insertstart + al;
				memmove( c_data + moveto, c_data + movestart, ( moveend - movestart ) * sizeof( Char ) );
				memcpy( c_data + insertstart, after.c_data, al * sizeof( Char ) );
				moveend = movestart - bl;
			}
		}
	}
	return *this;
}

int String::toInt( bool *ok, uint base ) const
{
	const Char *p = c_data;
	uint val = 0;
	uint l = length();
	const uint max_mult = INT_MAX / base;
	bool is_ok = false;
	bool neg = 0;
	uint dv = 0;

	if ( !p ) goto bye;

	while ( l && p->isSpace() ) // skip leading space
		l--, p++;

	if ( !l ) goto bye;
	if ( *p == '-' ) {
		l--;
		p++;
		neg = 1;
    }
	else if ( *p == '+' ) {
		l--;
		p++;
    }

    if ( !l || !ok_in_base( *p, base) ) goto bye;
	while ( l && ok_in_base( *p, base ) ) {
		l--;
		if ( p->isNumber() )
	    	dv = static_cast<uint>( p->digitValue() );
		if ( val > max_mult || ( val == max_mult && dv > ( INT_MAX % base ) + neg ) ) goto bye;
		val = base * val + dv;
		p++;
	}

	while ( l && p->isSpace() ) // skip trailing space
		l--, p++;
	if ( !l ) is_ok = true;

bye:
    if ( ok )
	*ok = is_ok;
    return is_ok ? ( neg ? -val : val ) : 0;
}

String &String::setNum( int n, uint base )
{
	char charbuf[33];
	char *buf = charbuf;
	char *p = &buf[32];
	int  len = 0;
	bool neg = false;
	if ( n < 0 ) {
		neg = true;
		n = -n;
	}
	do {
		*--p = "0123456789abcdefghijklmnopqrstuvwxyz"[( ( int )( n % base ) )];
		n /= base;
		++len;
	} while ( n );
    if ( neg ) {
		*--p = '-';
		++len;
	}
    Resize( len );
	memcpy( c_data, p, len );
	return *this;
}


String String::number( int n, uint base )
{
	String s;
	s.setNum( n, base );
	return s;
}

bool String::startsWith( const String &s ) const
{
	if ( isNull() ) return s.isNull();
	if ( s.length() > length() ) return false;
	const Char *c_this = c_data;
	const Char *c_s = s.c_data;
	const Char *c_s_end = s.c_data + s.length() - 1;
	while ( c_s <= c_s_end && *c_this == *c_s ) {
		c_this++;
		c_s++;
    }
	if ( c_s <= c_s_end ) return false;
	return ( *--c_this == *--c_s );
}

bool String::endsWith( const String &s ) const
{
	if ( isNull() ) return s.isNull();
	if ( s.length() > length() ) return false;
	const Char *c_this = ( c_data + length() - 1 );
	const Char *c_s = ( s.c_data + s.length() - 1 );
	while ( c_s >= s.c_data && *c_this == *c_s ) {
		c_this--;
		c_s--;
    }
	if ( c_s >= s.c_data ) return false;
	return ( *++c_this == *++c_s );
}

int String::compare( const String &s, bool cs ) const
{
	if ( !cs )
		return( ci_strcmp( *this, s ) );
	else
		return( cs_strcmp( *this, s ) );
}

int String::compare( const char *s, bool cs ) const
{
	if ( !cs )
		return( ci_strcmp( *this, String( s ) ) );
	else
		return( cs_strcmp( *this, String( s ) ) );
}

Char &String::operator[]( int pos )
{
	return at( pos );
}

String &String::operator+=( const String &str )
{
	return ( operator+=( str.data() ) );
}

String& String::operator+=( const char *str )
{
	if ( str ) {
		uint len2 = strlen( str );
		if ( len2 ) {
			uint len1 = length();
			Resize( len1 + len2 );
			strncpy( ( char * )( c_data + len1 ), str, len2 );
		}
	}
	return *this;
}

String &String::operator+=( Char c )
{
	Resize( length() + 1 );
	*( c_data + length() - 1 ) = c;
	return *this;
}

String &String::operator+=( char c )
{
	Resize( length() + 1 );
	*( c_data + length() - 1 ) = c;
	return *this;
}

String &String::operator=( Char c )
{
	Resize( 1 );
	*c_data = c;
	return *this;
}

String &String::operator=( char c )
{
	Resize( 1 );
	*c_data = c;
	return *this;
}

String &String::operator=( const char *str )
{
	int nbr_chars = strlen( str );
	Resize( nbr_chars ); // Check whether we need a different sized string
	strncpy( ( char * )c_data, str, nbr_chars );
	return *this;
}

String &String::operator=( const String &str )
{
	Resize( str.length() );
	memcpy( c_data, str.c_data, str.length() );
	return *this;
}
