	/***************************************************************************
                          string_class.h  -  description
                             -------------------
    begin                : Tue Jul 01 2003
    copyright            : (C) 2003 by Guilherme Fortunato
    email                : guilherme@linksat.com.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef STRING_H
	#define STRING_H

typedef unsigned char uchar;
typedef unsigned short ushort;

#include <cstdlib>

class Char
{

public:
	Char();
	Char( uchar );
	//Char( const Char & );
	Char( char );
	Char( int );
	//inline ~Char() { char_data = 0; }

	static Char null;

	bool isNull() const;
	bool isPrint() const;
	bool isPunct() const;
	bool isSpace() const;
	bool isLetter() const;
	bool isNumber() const;
	bool isLetterOrNumber() const;
	bool isSymbol() const;

	Char toLower() const;
	Char toUpper() const;

	int digitValue() const;
	int integerValue() const;

	inline char operator()( void )
	{
		return static_cast<char>( char_data );
	}

	friend inline bool operator==( char, Char );
	friend inline bool operator!=( char, Char );
	friend inline bool operator>=( char, Char );
	friend inline bool operator<=( char, Char );
	friend inline bool operator< ( char, Char );

	friend inline bool operator==( Char, char );
	friend inline bool operator!=( Char, char );
	friend inline bool operator>=( Char, char );
	friend inline bool operator<=( Char, char );
	friend inline bool operator< ( Char, char );

	friend inline bool operator==( Char, Char );
	friend inline bool operator!=( Char, Char );
	friend inline bool operator>=( Char, Char );
	friend inline bool operator<=( Char, Char );
	friend inline bool operator< ( Char, Char );

	Char &operator=( uchar );
	Char &operator=( char );
	Char &operator=( const Char & );

	inline const Char &operator=( char c ) const
	{
		return ( const_cast<Char *>( this ) )->operator=( c );
	}

	inline const Char &operator=( const Char &c ) const
	{
		return ( const_cast<Char *>( this ) )->operator=( c );
	}

	inline const Char &operator=( uchar c ) const
	{
		return ( const_cast<Char *>( this ) )->operator=( c );
	}

private:
	typedef enum Category
	{
		CHR_SPACE, CHR_NUMBER, CHR_LETTER, CHR_EXT_LETTER, CHR_PUNCT, CHR_SYMBOL, CHR_NONE
	};

	uchar char_data;

	bool belongs( Category, uchar ) const;
};

inline Char::Char()
{
	char_data = '\0';
}

inline Char::Char( uchar c )
{
	char_data = c;
}

inline Char::Char( char c )
{
	if ( c >= 0 )
		char_data = static_cast<uchar>( c );
	else
		char_data = 127 + static_cast<uchar>( ( -c ) );
}

inline Char::Char( int c )
{
	if ( c >= 0 && c <= 255 )
		char_data = static_cast<uchar>( c );
	else if ( c >= -127 && c < 0 )
		char_data = 127 + static_cast<uchar>( -c );
	else
		char_data = '\0';
}

inline bool Char::isPrint() const
{
	return( !belongs( CHR_NONE, char_data ) );
}

inline bool Char::isSpace() const
{
	return belongs( CHR_SPACE, char_data );
}

inline bool Char::isPunct() const
{
	return belongs( CHR_PUNCT, char_data );
}

inline bool Char::isLetter() const
{
	bool ret = true;
	if ( !belongs( CHR_LETTER, char_data ) ) {
		if ( !belongs( CHR_EXT_LETTER, char_data ) )
			ret = false;
	}
	return ret;
}

inline bool Char::isNumber() const
{
	return belongs( CHR_NUMBER, char_data );
}

inline bool Char::isLetterOrNumber() const
{
	bool ret = true;
	if ( !belongs( CHR_NUMBER, char_data ) ) {
		if ( !belongs( CHR_LETTER, char_data ) ) {
			if ( !belongs( CHR_EXT_LETTER, char_data ) )
				ret = false;
		}
	}
	return ret;
}

inline bool Char::isSymbol() const
{
	return belongs( CHR_SYMBOL, char_data );
}

inline int Char::digitValue() const
{
	if ( isNumber() )
		return ( static_cast<int>( char_data ) - 48 );
	return 0;
}

inline int Char::integerValue() const
{
	return ( static_cast<int>( char_data ) );
}

inline Char Char::toLower() const
{
	uchar c = char_data;
	if ( !belongs( CHR_LETTER, c ) ) {
		if ( belongs( CHR_EXT_LETTER, c ) )
			if ( c <= 220 ) c += 32;
	}
	else
		if ( c <= 90 ) c += 32;
	return Char( c );
}

inline Char Char::toUpper() const
{
	uchar c = char_data;
	if ( !belongs( CHR_LETTER, c ) ) {
		if ( belongs( CHR_EXT_LETTER, c ) )
			if ( c >= 224 ) c -= 32;
	}
	else
		if ( c >= 97 ) c -= 32;
	return Char( c );
}

inline Char &Char::operator=( char c )
{
	if ( c >= 0 )
		char_data = static_cast<uchar>( c );
	else
		char_data = 127 + static_cast<uchar>( ( -c ) );
	return *this;
}

inline Char &Char::operator=( const Char &c )
{
	char_data = c.char_data;
	return *this;
}

inline Char &Char::operator=( uchar c )
{
	char_data = c;
	return *this;
}

class String
{

public:
	String(); // make null string
	String( Char );                           // one-char string
	String( char );                           // one-char string
	String( const char * ); // deep copy
	String( const String & ); // impl-shared copy
	~String();

	static String null;

	bool isNull() const;
	bool isEmpty() const;
	uint length() const;
	void truncate( uint );

	void Resize( uint = 0 );

	String &fill( char, int = -1 );
	String &fill( Char, int = -1 );

	int find( char, int = 0, bool = false ) const;
	int find( const char *, int = 0, bool = false ) const;
	int find( Char, int = 0, bool = false ) const;
	int find( const String &, int = 0, bool = false ) const;

	int findRev( char, int = -1, bool = false ) const;
	int findRev( const char *, int = -1, bool = false ) const;
	int findRev( Char, int = -1, bool = false ) const;
	int findRev( const String &, int = -1, bool = false ) const;

	int contains( char, bool = false ) const;
	int contains( const char *, bool  = false ) const;
	int contains( Char, bool = false ) const;
	int contains( const String &, bool = false ) const;

	String &prepend( char );
	String &prepend( Char );
	String &prepend( const String & );
	String &prepend( const char * );

	String &append( char );
	String &append( Char );
	String &append( const String & );
	String &append( const char * );

	String arg( int, int = 0, uint = 10 ) const;
	String arg( uint, int = 0, uint = 10 ) const;
	String arg( char, int = 0 ) const;
	String arg( Char, int = 0 ) const;
	String arg( const char *, int = 0 ) const;
	String arg( const String &, int = 0 ) const;

	String left( uint ) const;
	String right( uint ) const;
	String mid( uint , uint = 1 ) const;

	String leftJustify( uint, Char = ' ', bool = false ) const;
	String rightJustify( uint, Char = ' ', bool = false ) const;

	String lower() const;
	String upper() const;

	String stripWhiteSpace() const;
	String &trimWhiteSpaces();

	String &insert( uint, char );
	String &insert( uint, Char );
	String &insert( uint, const char * );
	String &insert( uint, const String & );

	String &remove( char );
	String &remove( Char );
	String &remove( const char * );
	String &remove( const String & );

	String &replace( Char, Char );
	String &replace( uint, uint, const char *);
	String &replace( uint, uint, const String & );
	String &replace( uint, uint, char );
	String &replace( uint, uint, Char );
	String &replace( char, const String & );
	String &replace( Char, const String & );
	String &replace( const String &, const String & );

	int toInt( bool * = 0, uint = 10 ) const;
	String &setNum( int, uint = 10 );
	static String number( int, uint = 10 );

	int compare( const String &, bool = false ) const;
	int compare( const char *, bool = false ) const;

	bool startsWith( const String & ) const;
	bool endsWith( const String & ) const;

	Char &operator[]( int );
	inline const Char &operator[]( int pos ) const
	{
		return ( const_cast<String *>( this ) )->c_data[pos];
	}

	inline const char *data() const
	{
		return ( const char * )( c_data );
	}

	inline const Char *Data() const
	{
		return c_data;
	}

	String &operator+=( const String & );
	String &operator+=( const char * );
	String &operator+=( Char );
	String &operator+=( char );

	String &operator=( Char );
	String &operator=( char );
	String &operator=( const char * );      // deep copy
	String &operator=( const String & );   // impl-shared copy

	friend inline const String operator+( const String &, const String & );
	friend inline const String operator+( const String &, const char * );
	friend inline const String operator+( const char *, const String & );
	friend inline const String operator+( const String &, Char );
	friend inline const String operator+( const String &, char );
	friend inline const String operator+( Char, const String & );
	friend inline const String operator+( char, const String & );

	inline bool operator==( const String &s2 )
	{
		if ( compare( s2, true ) == 0 ) return true;
		return false;
	}

	inline bool operator==( const char *s2 )
	{
		if ( compare( s2, true ) == 0 ) return true;
		return false;
	}

	friend inline bool operator==( const String &, const String & );
	friend inline bool operator==( const String &, const char * );
	friend inline bool operator==( const char *, const String & );

	inline bool operator!=( const String &s2 )
	{
		if ( compare( s2, true ) != 0 ) return true;
		return false;
	}

	inline bool operator!=( const char *s2 )
	{
		if ( compare( s2, true ) != 0 ) return true;
		return false;
	}
	friend inline bool operator!=( const String &, const String & );
	friend inline bool operator!=( const String &, const char * );
	friend inline bool operator!=( const char *, const String & );

private:
	Char *c_data;
	uint n_chars;
	uint alloc_chars;

	void DeAlloc();
	String( uint );


	void DeepCopy( void *, uint );
	bool findArg( int &, int & ) const;
	String &insert( uint, const char *, uint );
	String &remove( uint, uint );
	String &replace( uint, uint, const char *, uint );
	Char &at( int );
};

inline String::String() : alloc_chars( 0 )
{
	c_data = NULL; // *c_data = c_data[0]
}

inline String::~String()
{
	DeAlloc();
}

inline bool operator==( char ch, Char c )
{
    return ( static_cast<uchar>( ch ) == c.char_data );
}

inline bool operator!=( char ch, Char c )
{
    return ( static_cast<uchar>( ch ) != c.char_data );
}

inline bool operator>=( char ch, Char c )
{
    return ( static_cast<uchar>( ch ) >= c.char_data );
}

inline bool operator<=( char ch, Char c )
{
    return ( static_cast<uchar>( ch ) <= c.char_data );
}

inline bool operator<( char ch, Char c )
{
    return ( static_cast<uchar>( ch ) < c.char_data );
}

inline bool operator==( Char c, char ch )
{
    return ( c.char_data == static_cast<uchar>( ch ) );
}

inline bool operator!=( Char c, char ch )
{
    return ( c.char_data != static_cast<uchar>( ch ) );
}

inline bool operator>=( Char c, char ch )
{
    return ( c.char_data >= static_cast<uchar>( ch ) );
}

inline bool operator<=( Char c, char ch )
{
    return ( c.char_data <= static_cast<uchar>( ch ) );
}

inline bool operator<( Char c, char ch )
{
    return ( c.char_data < static_cast<uchar>( ch ) );
}

inline bool operator==( Char ch, Char c )
{
    return ( ch.char_data == c.char_data );
}

inline bool operator!=( Char ch, Char c )
{
    return ( ch.char_data != c.char_data );
}

inline bool operator>=( Char ch, Char c )
{
    return ( ch.char_data >= c.char_data );
}

inline bool operator<=( Char ch, Char c )
{
    return ( ch.char_data <= c.char_data );
}

inline bool operator<( Char ch, Char c )
{
    return ( ch.char_data < c.char_data );
}

inline bool operator==( const String &s1, const String &s2 )
{
	if ( s1.compare( s2, true ) == 0 ) return true;
	return false;
}

inline bool operator==( const String &s1, const char *s2 )
{
	if ( s1.compare( s2, true ) == 0 ) return true;
	return false;
}

inline bool operator==( const char *s1, const String &s2 )
{
	if ( s2.compare( s1, true ) == 0 ) return true;
	return false;
}

inline bool operator!=( const String &s1, const String &s2 )
{
	if ( s1.compare( s2, true ) != 0 ) return true;
	return false;
}

inline bool operator!=( const String &s1, const char *s2 )
{
	if ( s1.compare( s2, true ) != 0 ) return true;
	return false;
}

inline bool operator!=( const char *s1, const String &s2 )
{
	if ( s2.compare( s1, true ) != 0 ) return true;
	return false;
}

inline const String operator+( const String &s1, const String &s2 )
	{
		String tmp( s1 );
		tmp += s2;
		return tmp;
	}

	inline const String operator+( const String &s1, const char *s2 )
	{
		String tmp( s1 );
		tmp += String( s2 );
		return tmp;
	}

	inline const String operator+( const char *s1, const String &s2 )
	{
		String tmp = String( s1 );
		tmp += s2;
		return tmp;
	}

	inline const String operator+( const String &s1, Char c2 )
	{
		String tmp( s1 );
		tmp += c2;
		return tmp;
	}

	inline const String operator+( const String &s1, char c2 )
	{
		String tmp( s1 );
		tmp += c2;
		return tmp;
	}

	inline const String operator+( Char c1, const String &s2 )
	{
		String tmp( c1 );
		tmp += s2;
		return tmp;
	}

	inline const String operator+( char c1, const String &s2 )
	{
		String tmp( c1 );
		tmp += s2;
		return tmp;
	}
#endif // STRING_H
